# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/
"""contains test for relation between axis and ftseries"""


__authors__ = ["H. Payno"]
__license__ = "MIT"
__date__ = "23/10/2019"


import unittest
from orangecontrib.tomwer.test.utils import OrangeWorflowTest
from tomwer.test.utils import skip_gui_test, skip_orange_workflows_test
from tomwer.core.utils.scanutils import MockHDF5, MockEDF
from silx.gui.utils.testutils import SignalListener
from tomwer.core.process.reconstruction.darkref.darkrefs import DarkRefs
from tomwer.core.process.reconstruction.axis.axis import AxisProcess
from tomwer.core.process.reconstruction.nabu.nabuslices import NabuSlices
import tempfile
import shutil
import time
import glob
import os
import h5py


TIMEOUT_TEST = 10


@unittest.skipIf(skip_orange_workflows_test(), reason="skip orange workflows tests")
@unittest.skipIf(skip_gui_test(), reason="skip gui test")
class TestAxisFtseriesEDF(OrangeWorflowTest):
    """Check the orange widget 'Axis' with an edf scan"""

    """
    Create the following workflow:
    datawatcherOW -> axisOW -> ftseriesOW
    """

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.nodeDataWatcher = cls.addWidget(
            cls, "orangecontrib.tomwer.widgets.control.DataWatcherOW.DataWatcherOW"
        )
        cls.nodeAxis = cls.addWidget(
            cls, "orangecontrib.tomwer.widgets.reconstruction.AxisOW.AxisOW"
        )
        cls.nodeFTSerie = cls.addWidget(
            cls, "orangecontrib.tomwer.widgets.reconstruction.FtseriesOW.FtseriesOW"
        )
        cls.dataSelectorNode = cls.addWidget(
            cls, "orangecontrib.tomwer.widgets.control.DataSelectorOW.DataSelectorOW"
        )

        # Let Orange process events (node creations)
        cls.processOrangeEvents(cls)

        # linking the workflow
        cls.link(cls, cls.nodeDataWatcher, "data", cls.nodeAxis, "data")
        cls.link(cls, cls.nodeAxis, "data", cls.nodeFTSerie, "data")
        cls.link(cls, cls.nodeFTSerie, "data", cls.dataSelectorNode, "data")
        # Let Orange process events (node creations)
        cls.processOrangeEvents(cls)

        # getting the widgets
        cls.datawatcherWidget = cls.getWidgetForNode(cls, cls.nodeDataWatcher)
        cls.axisWidget = cls.getWidgetForNode(cls, cls.nodeAxis)
        cls.ftserieWidget = cls.getWidgetForNode(cls, cls.nodeFTSerie)

    @classmethod
    def tearDownClass(cls):
        cls.datawatcherWidget.stopObservation()
        for node in (cls.nodeAxis, cls.nodeFTSerie, cls.nodeDataWatcher):
            cls.removeNode(cls, node)
        cls.app.processEvents()
        super().tearDownClass()

    def setUp(self) -> None:
        super().setUp()
        self.root_dir = tempfile.mkdtemp()
        self.scan_1_path = os.path.join(self.root_dir, "scan1")
        self.scan_2_path = os.path.join(self.root_dir, "scan2")
        self.scan_3_path = os.path.join(self.root_dir, "scan3")
        self.scan_dirs = [self.scan_1_path, self.scan_2_path, self.scan_3_path]

        self.mocks = []
        for scan_dir in self.scan_dirs:
            mock = MockEDF(scan_path=scan_dir, n_radio=20, n_ini_radio=20)
            assert os.path.exists(scan_dir)
            self.mocks.append(mock)

        # set up axis, ftseries and data watcher
        self.datawatcherWidget.setFolderObserved(self.root_dir)
        assert os.path.exists(self.root_dir)
        self.axisWidget._axis_params.mode = "manual"
        self.ftserieWidget._ftserie.dry_run = True
        self.ftserieWidget.recons_params.axis.do_axis_correction = True
        self.ftserieWidget.recons_params.axis.use_tomwer_axis = True
        self.ftserieWidget.recons_params.axis.use_old_tomwer_axis = False

        self.axisWidget._skip_exec(True)

        # create listeners
        self.axisListener = SignalListener()
        self.ftseriesListener = SignalListener()

        # signal / slot connection
        self.axisWidget.sigScanReady.connect(self.axisListener)
        self.ftserieWidget.sigScanReady.connect(self.ftseriesListener)

    def tearDown(self) -> None:
        shutil.rmtree(self.root_dir)
        super().tearDown()

    def testScenario(self):
        """
        scenario:
        1. scan1 is detected, axis should wait for an answer, then the axis is
           computed but not locked, ftseries should use the axis value computed.
        2. scan2 is detected, axis should wait for an answer, then axis is
           computed and lock this time, ftseries should use the axis value
           computed.
        3. scan3 is detected, it should pass directly through axis and the
           cor value set and used by ftseries
        """
        assert os.path.exists(self.scan_1_path)
        assert os.path.exists(self.scan_2_path)
        assert os.path.exists(self.scan_3_path)

        def _wait(timeout):
            timeout = timeout - 0.1
            self.app.processEvents()
            self.processOrangeEventsStack()
            time.sleep(0.1)
            return timeout

        # 1.
        self.datawatcherWidget.startObservation()
        self.axisWidget._axis_params.set_value_ref_tomwer(12.0)
        self.mocks[0].end_acquisition()

        # wait until we can validate
        timeout = TIMEOUT_TEST
        while timeout > 0 and self.axisWidget._n_skip == 0:
            timeout = _wait(timeout)
        if timeout < 0:
            raise TimeoutError("axis never ended calculation")
        validate_btn = self.axisWidget._widget._radioAxis._applyBut
        validate_btn.click()

        # wait until the axis windows expose (so scan should have been received)
        timeout = TIMEOUT_TEST
        while timeout > 0 and self.axisListener.callCount() < 1:
            timeout = _wait(timeout)
        if timeout < 0:
            raise TimeoutError("axis was never validated")

        timeout = TIMEOUT_TEST
        while timeout > 0 and self.ftseriesListener.callCount() < 1:
            timeout = _wait(timeout)
        if timeout < 0:
            raise TimeoutError("ftserie never ended reconstruction")

        scan_1_processes_file = os.path.join(self.scan_1_path, "tomwer_processes.h5")

        self.assertTrue(os.path.exists(scan_1_processes_file))
        with h5py.File(scan_1_processes_file, "r") as h5f:
            self.assertTrue("entry" in h5f)
            self.assertTrue("tomwer_process_0" in h5f["entry"])
            self.assertTrue("results" in h5f["entry"]["tomwer_process_0"])
            results = h5f["entry"]["tomwer_process_0"]["results"]
            self.assertTrue("center_of_rotation" in results)
            self.assertAlmostEqual(results["center_of_rotation"][()], 12.0)

        # 2.
        self.axisWidget._axis_params.set_value_ref_tomwer(-6.369)
        self.mocks[1].end_acquisition()

        # wait until we can validate
        timeout = TIMEOUT_TEST
        while timeout > 0 and self.axisWidget._n_skip == 1:
            timeout = _wait(timeout)
        if timeout < 0:
            raise TimeoutError("axis never ended calculation")
        validate_btn = self.axisWidget._widget._radioAxis._applyBut
        validate_btn.click()

        # wait until the axis windows expose (so scan should have been received)
        timeout = TIMEOUT_TEST
        while timeout > 0 and self.axisListener.callCount() < 2:
            timeout = _wait(timeout)
        if timeout < 0:
            raise TimeoutError("axis was never validated")

        timeout = TIMEOUT_TEST
        while timeout > 0 and self.ftseriesListener.callCount() < 2:
            timeout = _wait(timeout)
        if timeout < 0:
            raise TimeoutError("ftserie never ended reconstruction")
        scan_2_tomwer_processes = os.path.join(self.scan_2_path, "tomwer_processes.h5")
        self.assertTrue(os.path.exists(scan_2_tomwer_processes))
        with h5py.File(scan_2_tomwer_processes, "r") as h5f:
            self.assertTrue("entry" in h5f)
            self.assertTrue("tomwer_process_0" in h5f["entry"])
            self.assertTrue("results" in h5f["entry"]["tomwer_process_0"])
            results = h5f["entry"]["tomwer_process_0"]["results"]
            self.assertTrue("center_of_rotation" in results)
            self.assertAlmostEqual(results["center_of_rotation"][()], -6.369)

        # 3.
        # lock the result
        self.axisWidget.setLockValue(True)
        self.mocks[2].end_acquisition()

        # wait until the axis windows expose (so scan should have been received)
        timeout = TIMEOUT_TEST
        while timeout > 0 and self.axisListener.callCount() < 3:
            timeout = _wait(timeout)
        if timeout < 0:
            raise TimeoutError("axis was never validated")

        timeout = TIMEOUT_TEST
        while timeout > 0 and self.ftseriesListener.callCount() < 3:
            timeout = _wait(timeout)
        if timeout < 0:
            raise TimeoutError("ftserie never ended reconstruction")

        scan_3_tomwer_processes = os.path.join(self.scan_3_path, "tomwer_processes.h5")
        self.assertTrue(os.path.exists(scan_3_tomwer_processes))
        with h5py.File(scan_3_tomwer_processes, "r") as h5f:
            self.assertTrue("entry" in h5f)
            self.assertTrue("tomwer_process_0" in h5f["entry"])
            self.assertTrue("results" in h5f["entry"]["tomwer_process_0"])
            results = h5f["entry"]["tomwer_process_0"]["results"]
            self.assertTrue("center_of_rotation" in results)
            self.assertAlmostEqual(results["center_of_rotation"][()], -6.369)


@unittest.skipIf(skip_orange_workflows_test(), reason="skip orange workflows tests")
@unittest.skipIf(skip_gui_test(), reason="skip gui test")
class TestAxisFtseriesHDF5(OrangeWorflowTest):
    """Check the orange widget 'Axis' with an hdf5 scan"""

    """
    Create the following workflow:
    DataListOW -> DarkRefOW -> AxisOW -> nabuOW
    """

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.nodeDataList = cls.addWidget(
            cls, "orangecontrib.tomwer.widgets.control.DataListOW.DataListOW"
        )
        cls.nodeDarkRef = cls.addWidget(
            cls,
            "orangecontrib.tomwer.widgets.reconstruction.DarkRefAndCopyOW.DarkRefAndCopyOW",
        )
        cls.nodeAxis = cls.addWidget(
            cls, "orangecontrib.tomwer.widgets.reconstruction.AxisOW.AxisOW"
        )
        cls.nodeNabu = cls.addWidget(
            cls, "orangecontrib.tomwer.widgets.reconstruction.NabuOW.NabuOW"
        )

        # Let Orange process events (node creations)
        cls.processOrangeEvents(cls)

        # linking the workflow
        cls.link(cls, cls.nodeDataList, "data", cls.nodeDarkRef, "data")
        cls.link(cls, cls.nodeDarkRef, "data", cls.nodeAxis, "data")
        cls.link(cls, cls.nodeAxis, "data", cls.nodeNabu, "data")

        # Let Orange process events (node creations)
        cls.processOrangeEvents(cls)

        # getting the widgets
        cls.dataListWidget = cls.getWidgetForNode(cls, cls.nodeDataList)
        cls.darkRefWidget = cls.getWidgetForNode(cls, cls.nodeDarkRef)
        cls.axisWidget = cls.getWidgetForNode(cls, cls.nodeAxis)
        cls.nabuWidget = cls.getWidgetForNode(cls, cls.nodeNabu)

    @classmethod
    def tearDownClass(cls):
        for node in (cls.nodeDataList, cls.nodeDarkRef, cls.nodeAxis, cls.nodeNabu):
            cls.removeNode(cls, node)
        cls.app.processEvents()
        super().tearDownClass()

    def setUp(self) -> None:
        super().setUp()
        self.root_dir = tempfile.mkdtemp()
        self.scan_1_path = os.path.join(self.root_dir, "scan1")
        self.scan_2_path = os.path.join(self.root_dir, "scan2")
        self.scan_3_path = os.path.join(self.root_dir, "scan3")
        scan_dirs = (self.scan_1_path, self.scan_2_path, self.scan_3_path)

        self.mocks = []
        self.scans = []
        for scan_dir in scan_dirs:
            mock = MockHDF5(scan_path=scan_dir, n_proj=20, n_ini_proj=20)
            assert os.path.exists(scan_dir)
            self.mocks.append(mock)
            self.scans.append(mock.scan)

        # set up axis, ftseries and data watcher
        for mock in self.mocks:
            self.dataListWidget.add(mock.scan)

        self.axisWidget._axis_params.mode = "manual"
        self.nabuWidget.setDryRun(True)
        # only ask for the creaation of the configuration file for the volume
        self.nabuWidget.setConfiguration({"tomwer_slices": None})

        # monkey patch exec_ to check behavior
        self.nabuWidget._replaceExec_()

        self.axisWidget._skip_exec(True)
        self.axisWidget.setModeLock("global")

        # create listeners
        self.axisListener = SignalListener()
        self.nabuListener = SignalListener()
        self.darkrefListener = SignalListener()

        # signal / slot connection
        self.axisWidget.sigScanReady.connect(self.axisListener)
        self.nabuWidget.sigScanReady.connect(self.nabuListener)
        self.darkRefWidget.sigScanReady.connect(self.darkrefListener)

    def tearDown(self) -> None:
        shutil.rmtree(self.root_dir)
        super().tearDown()

    def testScenario(self):
        """
        scenario:
        scan1, scan2 and scan 3 are all send to the pipeline
        DarkRef, axis, nabu.
        Lock the reconstruction method to global.
        Insure:
         - those widgets are able to work together
         - dark ref is created
         - axis is computed and value pass to nabu.
         - those processes update the tomwer_processes file
        """
        assert os.path.exists(self.scan_1_path)
        assert os.path.exists(self.scan_2_path)
        assert os.path.exists(self.scan_3_path)

        def _wait(timeout):
            timeout = timeout - 0.1
            self.app.processEvents()
            self.processOrangeEventsStack()
            time.sleep(0.1)
            return timeout

        for scan in self.scans:
            self.assertFalse(os.path.exists(scan.process_file))

        # 1. start workflow
        self.dataListWidget._sendList()

        # 2. check dark ref
        # wait until all scan pass dark ref process
        timeout = TIMEOUT_TEST
        while timeout > 0 and self.darkrefListener.callCount() < 3:
            timeout = _wait(timeout)
        if timeout < 0:
            raise TimeoutError("dark ref listener didn't received all the " "scans")

        # 2.1 make sure DarkRef process has been registered in tomwer_processes.h5
        # and a value has been saved and are valid
        for scan in self.scans:
            self.assertTrue(os.path.exists(scan.process_file))
            self.assertEqual(
                len(
                    DarkRefs.get_darks_frm_process_file(
                        process_file=scan.process_file, entry=scan.entry
                    )
                ),
                1,
            )
            self.assertEqual(
                len(
                    DarkRefs.get_flats_frm_process_file(
                        process_file=scan.process_file, entry=scan.entry
                    )
                ),
                1,
            )

        # 3. check axis
        # wait until all scan pass axis process
        timeout = TIMEOUT_TEST
        while timeout > 0 and self.axisListener.callCount() < 3:
            timeout = _wait(timeout)
        if timeout < 0:
            raise TimeoutError("axis listener didn't received all the " "scans")
        # 3.1 make sure Axis process has been registered in tomwer_processes.h5
        # and a value has been saved
        for i_scan, scan in enumerate(self.scans):
            cor_value = AxisProcess.get_cor_frm_process_file(
                process_file=scan.process_file, entry=scan.entry
            )
            self.assertTrue(cor_value is not None)
            self.assertTrue(-scan.dim_1 / 2 <= cor_value <= scan.dim_1 / 2)

        # 4. check nabu
        # wait until all scan pass nabu process

        timeout = TIMEOUT_TEST
        while timeout > 0 and self.nabuListener.callCount() < 3:
            timeout = _wait(timeout)
        if timeout < 0:
            raise TimeoutError("nabu listener didn't received all the " "scans")

        # 4.1 make sure each scan contains now a nabu .cfg file
        for scan in self.scans:
            self.assertEqual(len(glob.glob(os.sep.join((scan.path, "*.cfg")))), 1)

        # 4.2 make sure Nabu process has been registered in tomwer_processes.h5
        for scan in self.scans:
            nabu_proc_rec = NabuSlices.get_process_frm_process_file(
                process_file=scan.process_file, entry=scan.entry
            )
            self.assertTrue(nabu_proc_rec is not None)


def suite():
    test_suite = unittest.TestSuite()
    for ui in (TestAxisFtseriesEDF, TestAxisFtseriesHDF5):
        test_suite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(ui))
    return test_suite



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/

__authors__ = ["H. Payno"]
__license__ = "MIT"
__date__ = "04/03/2019"

from silx.gui import qt
from ...utils.flow import FlowDirection
from .nabuconfig import NabuConfiguration
from tomwer.core.process.reconstruction.nabu.utils import _NabuMode, _ConfigurationLevel
from tomwer.gui.reconstruction.nabu.nabuflow import NabuFlowArea
from tomoscan.scanbase import _FOV
from silx.utils.enum import Enum as _Enum
from tomwer.gui import icons as tomwer_icons
from .action import BasicConfigurationAction, ExpertConfigurationAction, FilterAction


class _NabuStages(_Enum):
    INI = "initialization"
    PRE = "pre-processing"
    PHASE = "phase"
    PROC = "processing"
    POST = "post-processing"
    VOLUME = "volume"

    @staticmethod
    def getStagesOrder():
        return (
            _NabuStages.INI,
            _NabuStages.PRE,
            _NabuStages.PHASE,
            _NabuStages.PROC,
            _NabuStages.POST,
        )

    @staticmethod
    def getProcessEnum(stage):
        """Return the process Enum associated to the stage"""
        stage = _NabuStages.from_value(stage)
        if stage is _NabuStages.INI:
            raise NotImplementedError()
        elif stage is _NabuStages.PRE:
            return _NabuPreprocessing
        elif stage is _NabuStages.PHASE:
            return _NabuPhase
        elif stage is _NabuStages.PROC:
            return _NabuProcessing
        elif stage is _NabuStages.POST:
            raise NotImplementedError()
        raise NotImplementedError()


class _NabuPreprocessing(_Enum):
    """Define all the preprocessing action possible and the order they
    are applied on"""

    FLAT_FIELD_NORMALIZATION = "flat field normalization"
    CCD_FILTER = "hot spot correction"

    @staticmethod
    def getPreProcessOrder():
        return (
            _NabuPreprocessing.FLAT_FIELD_NORMALIZATION,
            _NabuPreprocessing.CCD_FILTER,
        )


class _NabuPhase(_Enum):
    """Define all the phase action possible and the order they
    are applied on"""

    PHASE = "phase retrieval"
    UNSHARP_MASK = "unsharp mask"
    LOGARITHM = "logarithm"

    @staticmethod
    def getPreProcessOrder():
        return (_NabuPhase.PHASE, _NabuPhase.UNSHARP_MASK, _NabuPhase.LOGARITHM)


class _NabuProcessing(_Enum):
    """Define all the processing action possible"""

    RECONSTRUCTION = 0

    @staticmethod
    def getProcessOrder():
        return _NabuProcessing.RECONSTRUCTION


class _NabuProcess(qt.QWidget):
    def __init__(self, parent):
        qt.QWidget.__init__(self, parent=parent)
        self.setLayout(qt.QGridLayout())
        self._stageCB = qt.QComboBox(parent=self)
        for stage in _NabuStages.values():
            self._stageCB.addItem(stage)

        self.layout().addWidget(qt.QLabel("stage:", self), 0, 0, 1, 1)
        self.layout().addWidget(self._stageCB, 0, 1, 1, 1)

        self._configurationWidget = _NabuConfiguration(parent=parent)
        self.layout().addWidget(self._configurationWidget, 1, 0, 2, 2)


class _NabuConfiguration(qt.QWidget):
    def __init__(self, parent):
        qt.QWidget.__init__(self, parent=parent)
        self.setLayout(qt.QFormLayout)

    def filter(self, stage=None, process=None):
        """Apply a filter on the options to show

        :param str stage:
        :param str process:
        """
        pass

    def setStages(self, stages: dict) -> None:
        """

        :param dict stages: contains stages the user can edit and for each
                            stages the associated processes.
        """
        self.clear()
        for stage, processes in stages.items():
            self.addStage(stage=stage, processes=processes)

    def addStage(self, stage, processes):
        stage = _NabuStages.from_value(value=stage)
        for process in processes:
            _NabuStages.getProcessEnum(stage=stage)


class NabuDialog(qt.QDialog):
    sigComputationRequested = qt.Signal()
    """Signal emitted when a computation is requested"""

    def __init__(self, parent):
        qt.QDialog.__init__(self, parent)
        self.setLayout(qt.QVBoxLayout())

        self._widget = NabuWindow(self)
        self.layout().addWidget(self._widget)

        self._computePB = qt.QPushButton("compute", self)
        self._buttons = qt.QDialogButtonBox(self)
        self._buttons.addButton(self._computePB, qt.QDialogButtonBox.ActionRole)
        self.layout().addWidget(self._buttons)

        # set up

        # expose API
        self.setOutputDir = self._widget.setOutputDir
        self.setScan = self._widget.setScan

        # connect signal / slot

    def getConfiguration(self):
        return self._widget.getConfiguration()


class NabuWindow(qt.QMainWindow):

    sigConfigChanged = qt.Signal()
    """Signal emitted when the configuration change"""

    def __init__(self, parent, flow_direction="vertical"):
        qt.QMainWindow.__init__(self, parent=parent)
        self.setWindowFlags(qt.Qt.Widget)

        self._mainWidget = NabuWidget(parent=self, flow_direction=flow_direction)
        self.setCentralWidget(self._mainWidget)

        # add toolbar
        toolbar = qt.QToolBar(self)
        toolbar.setMovable(False)
        toolbar.setFloatable(False)
        self.addToolBar(qt.Qt.TopToolBarArea, toolbar)

        # add filtering
        self._filterAction = FilterAction(toolbar)
        toolbar.addAction(self._filterAction)
        self._filterAction.triggered.connect(self._filteringChanged)

        # add configuration mode
        self.__configurationModesAction = qt.QAction(self)
        self.__configurationModesAction.setCheckable(False)
        menu = qt.QMenu(self)
        self.__configurationModesAction.setMenu(menu)
        toolbar.addAction(self.__configurationModesAction)

        self.__configurationModesGroup = qt.QActionGroup(self)
        self.__configurationModesGroup.setExclusive(True)
        self.__configurationModesGroup.triggered.connect(self._userModeChanged)

        self._basicConfigAction = BasicConfigurationAction(toolbar)
        menu.addAction(self._basicConfigAction)
        self.__configurationModesGroup.addAction(self._basicConfigAction)
        self._expertConfiguration = ExpertConfigurationAction(toolbar)
        menu.addAction(self._expertConfiguration)
        self.__configurationModesGroup.addAction(self._expertConfiguration)

        # expose API
        self.getConfiguration = self._mainWidget.getConfiguration
        self.setConfiguration = self._mainWidget.setConfiguration
        self.setOutputDir = self._mainWidget.setOutputDir
        self.setScan = self._mainWidget.setScan
        self.getMode = self._mainWidget.getMode
        self.setMode = self._mainWidget.setMode

        # connect signal / slot
        self._mainWidget.sigConfigChanged.connect(self._triggerConfigChanged)

        # set up
        self._filterAction.setChecked(True)
        self._basicConfigAction.setChecked(True)
        self._userModeChanged()

    def _userModeChanged(self, *args, **kwargs):
        selectedAction = self.__configurationModesGroup.checkedAction()
        self.__configurationModesAction.setIcon(selectedAction.icon())
        self.__configurationModesAction.setToolTip(selectedAction.tooltip())
        self._mainWidget.setConfigurationLevel(self.getConfigurationLevel())

    def _filteringChanged(self, *args, **kwargs):
        self._mainWidget.setFilteringActive(self.isFilteringActive())

    def isFilteringActive(self):
        return self._filterAction.isChecked()

    def getConfigurationLevel(self):
        if self._basicConfigAction.isChecked():
            return _ConfigurationLevel.OPTIONAL
        elif self._expertConfiguration.isChecked():
            return _ConfigurationLevel.ADVANCED
        else:
            raise ValueError("Level not recognize")

    def _triggerConfigChanged(self):
        self.sigConfigChanged.emit()


class NabuWidget(qt.QWidget):
    """
    Widget containing the entire gui for nabu (control flow + parameters
    settings)
    """

    sigConfigChanged = qt.Signal()
    """Signal emitted when the configuration change"""

    def __init__(self, parent, flow_direction="vertical"):
        qt.QWidget.__init__(self, parent=parent)
        flow_direction = FlowDirection.from_value(flow_direction)
        self.setLayout(qt.QGridLayout())
        self._filteringActive = True
        self._configuration_level = _ConfigurationLevel.OPTIONAL

        # reconstruction type
        self._widget_recons = qt.QWidget(parent=self)
        self._widget_recons.setLayout(qt.QHBoxLayout())
        self._modeLabel = qt.QLabel("mode:")
        self._widget_recons.layout().addWidget(self._modeLabel)
        self._modeLabel.setSizePolicy(qt.QSizePolicy.Minimum, qt.QSizePolicy.Minimum)
        self._nabuModeCB = qt.QComboBox(parent=self)
        for mode in _NabuMode:
            self._nabuModeCB.addItem(mode.value)
        self._widget_recons.layout().addWidget(self._nabuModeCB)
        self.layout().addWidget(self._widget_recons, 0, 0, 1, 2)

        # reconstruction type
        self._widget_recons = qt.QWidget(parent=self)
        self._widget_recons.setLayout(qt.QHBoxLayout())
        self._widget_recons.layout().addWidget(qt.QLabel("mode:"))
        self._nabuModeCB = qt.QComboBox(parent=self)
        for mode in _NabuMode:
            self._nabuModeCB.addItem(mode.value)
        self._widget_recons.layout().addWidget(self._nabuModeCB)
        self.layout().addWidget(self._widget_recons, 0, 0, 1, 2)

        # flow
        self._flow = NabuFlowArea(parent=self, direction=flow_direction)
        self._flow.setSizePolicy(qt.QSizePolicy.Minimum, qt.QSizePolicy.Minimum)
        self.layout().addWidget(self._flow, 1, 0, 5, 1)

        # nabu configuration
        self._configurationScrollArea = qt.QScrollArea(self)
        self._configurationScrollArea.setWidgetResizable(True)
        self._configuration = NabuConfiguration(parent=self)
        self._configurationScrollArea.setWidget(self._configuration)
        self._configurationScrollArea.setHorizontalScrollBarPolicy(
            qt.Qt.ScrollBarAlwaysOff
        )
        self.layout().addWidget(self._configurationScrollArea, 1, 1, 5, 1)
        self._configuration.setSizePolicy(
            qt.QSizePolicy.Expanding, qt.QSizePolicy.Expanding
        )
        self._configurationScrollArea.setSizePolicy(
            qt.QSizePolicy.Expanding, qt.QSizePolicy.Expanding
        )

        # expose API
        self.setIniProcessing = self._flow.setIniProcessing
        self.setPreProcessing = self._flow.setPreProcessing
        self.setPhaseProcessing = self._flow.setPhaseProcessing
        self.setProcessing = self._flow.setProcessing
        self.setPostProcessing = self._flow.setPostProcessing

        self.setOutputDir = self._configuration.setOutputDir
        self.getActiveProcess = self._flow.getProcessFocused

        # set up
        pre_processing = ("pre processing",)
        roman_one_icon = tomwer_icons.getQIcon("roman_one")
        self.setPreProcessing(pre_processing, icons=(roman_one_icon,))
        phase_processing = ("phase",)
        roman_two_icon = tomwer_icons.getQIcon("roman_two")
        phase_icons = (roman_two_icon,)
        self.setPhaseProcessing(phase_processing, icons=phase_icons)
        processing = ("reconstruction",)
        processing_icons = (tomwer_icons.getQIcon("roman_three"),)
        self.setProcessing(processes=processing, icons=processing_icons)
        post_processing = ("save",)
        post_processing_icons = (tomwer_icons.getQIcon("roman_four"),)
        self.setPostProcessing(post_processing, icons=post_processing_icons)
        index_mode = self._nabuModeCB.findText(_NabuMode.FULL_FIELD.value)
        assert index_mode >= 0, "full filed should be registered in the widget"
        self._nabuModeCB.setCurrentIndex(index_mode)
        self._flow.setMaximumWidth(240)

        # signal / slot connections
        self._flow.sigConfigurationChanged.connect(self._processSelectionChanged)
        self._flow.sigResetConfiguration.connect(self._processSelectionChanged)
        self._configuration.sigConfChanged.connect(self._triggerConfigchanged)
        self._nabuModeCB.currentIndexChanged.connect(self._triggerConfigchanged)

    def _triggerConfigchanged(self, *args, **kwargs):
        self.sigConfigChanged.emit()

    def setScan(self, scan):
        """Tune the configuration from scan information if possible"""
        if scan.field_of_view is not None:
            if scan.field_of_view == _FOV.FULL:
                self.setMode(_NabuMode.FULL_FIELD)
            elif scan.field_of_view == _FOV.HALF:
                self.setMode(_NabuMode.HALF_ACQ)

    def getConfiguration(self):
        conf = self._configuration.getConfiguration()
        enable_ht = int(self.getMode() is _NabuMode.HALF_ACQ)
        conf["reconstruction"]["enable_halftomo"] = enable_ht
        return conf

    def setConfiguration(self, config):
        if "reconstruction" in config and "enable_halftomo" in config["reconstruction"]:
            if config["reconstruction"]["enable_halftomo"] == 1:
                index = self._nabuModeCB.findText(_NabuMode.HALF_ACQ.value)
            else:
                index = self._nabuModeCB.findText(_NabuMode.FULL_FIELD.value)
        self._nabuModeCB.setCurrentIndex(index)
        self._configuration.setConfiguration(config=config)

    def getMode(self):
        return _NabuMode.from_value(self._nabuModeCB.currentText())

    def setMode(self, mode):
        mode = _NabuMode.from_value(mode)
        idx = self._nabuModeCB.findText(mode.value)
        self._nabuModeCB.setCurrentIndex(idx)

    def _processSelectionChanged(self, *arg):
        if self.isConfigFiltered():
            self.updateConfigurationFilter()

    def isConfigFiltered(self):
        return self._filteringActive

    def updateConfigurationFilter(self, *args, **kwargs):
        if self.isConfigFiltered():
            stage, option = self.getActiveProcess()
        else:
            stage = None
            option = None
        self._configuration.applyFilter(stage=stage, option=option)
        self._configuration.setConfigurationLevel(self.getConfigurationLevel())
        # force scroll bar to update
        self._configurationScrollArea.updateGeometry()

    def setConfigurationLevel(self, level):
        level = _ConfigurationLevel.from_value(level)
        self._configuration_level = level
        self.updateConfigurationFilter()

    def getConfigurationLevel(self):
        return self._configuration_level

    def setFilteringActive(self, active):
        self._filteringActive = active
        self.updateConfigurationFilter()



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/

__authors__ = ["H. Payno"]
__license__ = "MIT"
__date__ = "19/07/2018"

from silx.gui import qt
from Orange.widgets import widget, gui
from Orange.widgets.settings import Setting
from Orange.widgets.widget import Input, Output
from tomwer.core.process.control.timer import Timer
from tomwer.core.scan.scanbase import TomwerScanBase
import logging

import functools

_logger = logging.getLogger(__name__)


class _TimerWidget(qt.QWidget):
    def __init__(self, parent, _time=None):
        if _time is not None:
            assert type(_time) is int
        qt.QWidget.__init__(self, parent)
        self.setLayout(qt.QGridLayout())

        self.layout().addWidget(qt.QLabel("time to wait (in sec):", parent=self), 0, 0)
        self._timeLE = qt.QSpinBox(parent=self)
        self._timeLE.setMinimum(0)
        self._timeLE.setValue(_time or 1)
        self.layout().addWidget(self._timeLE, 0, 1)

        spacer = qt.QWidget(parent=self)
        spacer.setSizePolicy(qt.QSizePolicy.Minimum, qt.QSizePolicy.Expanding)
        self.layout().addWidget(spacer, 1, 0)

        # expose API
        self.timeChanged = self._timeLE.valueChanged


class TimerOW(widget.OWWidget, Timer):
    name = "timer"
    id = "orange.widgets.tomwer.filterow"
    description = (
        "Simple widget which wait for a defined amont of time and" "release the data"
    )
    icon = "icons/time.png"
    priority = 200
    category = "esrfWidgets"
    keywords = ["control", "timer", "wait", "data"]

    want_main_area = True
    resizing_enabled = True
    compress_signal = False
    _waiting_time = Setting(int(1))

    class Inputs:
        data_in = Input(name="data", type=TomwerScanBase)

    class Outputs:
        data_out = Output(name="data", type=TomwerScanBase)

    def __init__(self, parent=None):
        """"""
        widget.OWWidget.__init__(self, parent)
        Timer.__init__(self, wait=self._waiting_time)
        self._widget = _TimerWidget(parent=self, _time=self._waiting_time)
        self._widget.setContentsMargins(0, 0, 0, 0)

        layout = gui.vBox(self.mainArea, self.name).layout()
        layout.addWidget(self._widget)

        self._widget.timeChanged.connect(self._updateTime)

    @Inputs.data_in
    def process(self, scan):
        if scan is None:
            return
        callback = functools.partial(self.Outputs.data_out.send, scan)
        _timer = qt.QTimer(self)
        _timer.singleShot(self._waiting_time * 1000, callback)

    def _updateTime(self, newTime):
        self._waiting_time = newTime



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/

__authors__ = ["C. Nemoz", "H. Payno"]
__license__ = "MIT"
__date__ = "01/12/2016"

import logging
import os
from collections import OrderedDict
from silx.gui import qt
from tomwer.core.process.control.scanlist import _ScanList
from tomwer.core.utils import logconfig
from tomwer.gui.qfolderdialog import QScanDialog
from nxtomomill import converter as nxtomomill_converter
from tomwer.core.scan.hdf5scan import HDF5TomoScan
from tomwer.core.scan.scanbase import TomwerScanBase
from tomwer.core.scan.blissscan import BlissScan
from tomwer.core.scan.scanfactory import ScanFactory

logger = logging.getLogger(__name__)


class _DataListDialog(qt.QWidget):
    """A simple list of dataset path.

    .. warning: the widget won't check for scan validity and will only
        emit the path to folders to the next widgets

    :param parent: the parent widget
    """

    def __init__(self, parent=None):
        qt.QWidget.__init__(self, parent)
        self.setLayout(qt.QVBoxLayout())
        # add list
        self.datalist = self.createDataList()
        self.layout().addWidget(self.datalist)
        # add buttons
        self._buttons = qt.QDialogButtonBox(parent=self)
        self._addButton = qt.QPushButton("Add", parent=self)
        self._buttons.addButton(self._addButton, qt.QDialogButtonBox.ActionRole)
        self._rmButton = qt.QPushButton("Remove", parent=self)
        self._buttons.addButton(self._rmButton, qt.QDialogButtonBox.ActionRole)
        self._rmAllButton = qt.QPushButton("Remove all", parent=self)
        self._buttons.addButton(self._rmAllButton, qt.QDialogButtonBox.ActionRole)

        self._sendButton = qt.QPushButton("Send", parent=self)
        self._buttons.addButton(self._sendButton, qt.QDialogButtonBox.AcceptRole)
        self.layout().addWidget(self._buttons)

        # expose API
        self._sendList = self.datalist._sendList
        self.add = self.datalist.add
        self.remove = self.datalist.remove
        self.setScanIDs = self.datalist.setScanIDs
        self._scheme_title = self.datalist._scheme_title
        self.length = self.datalist.length
        self.selectAll = self.datalist.selectAll
        self.clear = self.datalist.clear

        # connect signal / slot
        self._addButton.clicked.connect(self._callbackAddPath)
        self._rmButton.clicked.connect(self._callbackRemoveFolder)
        self._rmAllButton.clicked.connect(self._callbackRemoveAllFolders)

    def n_scan(self):
        return len(self.datalist.items)

    def _callbackAddPath(self):
        """"""
        raise NotImplementedError("Base class")

    def _callbackRemoveFolder(self):
        """"""
        selectedItems = self.datalist.selectedItems()
        toRemove = []
        if selectedItems is not None and len(selectedItems) > 0:
            for item in selectedItems:
                toRemove.append(item.text())
        for scan in toRemove:
            self.remove(scan)

    def _callbackRemoveAllFolders(self):
        self.datalist.selectAll()
        self._callbackRemoveFolder()

    def createDataList(self):
        raise NotImplementedError("Base class")


class BlissHDF5DataListDialog(_DataListDialog):
    """Dialog used to load .h5 only (used when for nxtomomillOW when we need)
    to do a conversion from bliss.h5 to NXtomo"""

    def __init__(self, parent):
        _DataListDialog.__init__(self, parent)
        self._sendButton.setText("Send all")
        self._sendSelectedButton = qt.QPushButton("Send selected", self)
        self._buttons.addButton(
            self._sendSelectedButton, qt.QDialogButtonBox.AcceptRole
        )

    def createDataList(self):
        return BlissDataList(self)

    def _callbackAddPath(self):
        """"""
        dialog = qt.QFileDialog(self)
        dialog.setNameFilters(["HDF5 file *.h5 *.hdf5 *.nx *.nexus"])

        if not dialog.exec_():
            dialog.close()
            return

        filesSelected = dialog.selectedFiles()
        for file_ in filesSelected:
            self.add(file_)


class GenericDataListDialog(_DataListDialog):
    """Dialog used to load EDFScan or HEDF5 scans"""

    def createDataList(self):
        return GenericDataList(self)

    def _callbackAddPath(self):
        """"""
        dialog = QScanDialog(self, multiSelection=True)

        if not dialog.exec_():
            dialog.close()
            return

        files_or_folders = dialog.files_selected()
        for file_or_folder in files_or_folders:
            self.add(file_or_folder)


class _DataList(_ScanList, qt.QTableWidget):
    def __init__(self, parent):
        _ScanList.__init__(self)
        qt.QTableWidget.__init__(self, parent)
        self.setRowCount(0)
        self.setColumnCount(1)
        self.setSortingEnabled(True)
        self.verticalHeader().hide()
        if hasattr(self.horizontalHeader(), "setSectionResizeMode"):  # Qt5
            self.horizontalHeader().setSectionResizeMode(0, qt.QHeaderView.Stretch)
        else:  # Qt4
            self.horizontalHeader().setResizeMode(0, qt.QHeaderView.Stretch)
        self.setAcceptDrops(True)
        self.items = OrderedDict()

    def remove_item(self, item):
        """Remove a given folder"""
        del self.items[item.text()]
        itemIndex = self.row(item)
        self.takeItem(itemIndex, 0)
        _ScanList.remove(self, item.text())
        self.removeRow(item.row())
        self.setRowCount(self.rowCount() - 1)
        self._update()

    def remove(self, scan):
        if scan is not None and scan in self.items:
            item = self.items[scan]
            itemIndex = self.row(item)
            self.takeItem(itemIndex, 0)
            _ScanList.remove(self, scan)
            del self.items[scan]
            self.removeRow(item.row())
            self.setRowCount(self.rowCount() - 1)
            self._update()

    def _update(self):
        list_scan = list(self.items.keys())
        self.clear()
        for scan in list_scan:
            self.add(scan)
        self.sortByColumn(0, self.horizontalHeader().sortIndicatorOrder())

    def add(self, d):
        """add the folder or path"""
        raise NotImplementedError("Base class")

    def _addScanIDItem(self, d):
        if "@" not in d and not os.path.isdir(d):
            warning = (
                "Skipping the observation of %s, directory not existing on the system"
                % d
            )
            logger.info(warning, extra={logconfig.DOC_TITLE: self._scheme_title})
        elif d in self.items:
            logger.debug("The path {} is already in the scan list".format(d))
        else:
            row = self.rowCount()
            self.setRowCount(row + 1)

            _item = qt.QTableWidgetItem()
            _item.setText(d)
            _item.setFlags(qt.Qt.ItemIsEnabled | qt.Qt.ItemIsSelectable)
            self.setItem(row, 0, _item)

            self.items[d] = _item

    def setScanIDs(self, scanIDs):
        [self._addScanIDItem(item) for item in scanIDs]
        _ScanList.setScanIDs(self, scanIDs)

    def clear(self):
        """Remove all items on the list"""
        self.items = OrderedDict()
        _ScanList.clear(self)
        qt.QTableWidget.clear(self)
        self.setRowCount(0)
        self.setHorizontalHeaderLabels(["folder"])
        if hasattr(self.horizontalHeader(), "setSectionResizeMode"):  # Qt5
            self.horizontalHeader().setSectionResizeMode(0, qt.QHeaderView.Stretch)
        else:  # Qt4
            self.horizontalHeader().setResizeMode(0, qt.QHeaderView.Stretch)

    def dropEvent(self, event):
        if event.mimeData().hasFormat("text/uri-list"):
            for url in event.mimeData().urls():
                self.add(str(url.path()))

    def supportedDropActions(self):
        """Inherited method to redefine supported drop actions."""
        return qt.Qt.CopyAction | qt.Qt.MoveAction

    def dragEnterEvent(self, event):
        if event.mimeData().hasFormat("text/uri-list"):
            event.accept()
            event.setDropAction(qt.Qt.CopyAction)
        else:
            qt.QListWidget.dragEnterEvent(self, event)

    def dragMoveEvent(self, event):
        if event.mimeData().hasFormat("text/uri-list"):
            event.setDropAction(qt.Qt.CopyAction)
            event.accept()
        else:
            qt.QListWidget.dragMoveEvent(self, event)


class GenericDataList(_DataList):
    """Data list able to manage directories (EDF/HDF5?) or files (HDF5)"""

    def __init__(self, parent):
        _DataList.__init__(self, parent)
        self.setHorizontalHeaderLabels(["scan"])

    def add(self, path):
        """Add the path folder d in the scan list

        :param d: the path of the directory to add
        :type d: Union[str, TomoBase]
        """
        if isinstance(path, TomwerScanBase):
            scan_obj = _ScanList.add(self, path)
            self._addScanIDItem(str(scan_obj))
        elif os.path.isdir(path):
            scan_obj = _ScanList.add(self, path)
            if scan_obj:
                self._addScanIDItem(str(scan_obj.path))
        else:
            # try to add it as HDF5TomoScan
            try:
                scan_objs = ScanFactory.create_scan_objects(path)
            except Exception as e:
                logger.error(e)
            else:
                if scan_objs is None:
                    return
                for scan in scan_objs:
                    scan_obj = _ScanList.add(self, scan)
                    if scan_obj:
                        self._addScanIDItem(str(scan_obj))


class BlissDataList(_DataList):
    def __init__(self, parent):
        _DataList.__init__(self, parent)
        self.setHorizontalHeaderLabels(["entry@bliss_file.h5"])

    def add(self, path):
        """Add the path folder d in the scan list

        :param d: the path of the directory to add
        :type d: Union[str, TomoBase]
        """
        if "@" in path:
            entry, path = path.split("@")
            possible_entries = [entry]
        else:
            if not BlissScan.is_bliss_file(path):
                msg = qt.QMessageBox(self)
                msg.setIcon(qt.QMessageBox.Warning)
                types = qt.QMessageBox.Ok | qt.QMessageBox.Cancel
                msg.setStandardButtons(types)

                if HDF5TomoScan.is_nexus_nxtomo_file(path):
                    text = (
                        "The input file `{}` seems to contain `NXTomo` entries. "
                        "and no valid `Bliss` valid entry. \n"
                        "This is probably not a Bliss file. Do you still want to "
                        "translate ?".format(path)
                    )
                else:
                    text = (
                        "The input file `{}` does not seems to contain any"
                        "valid `Bliss` entry. \n"
                        "This is probably not a Bliss file. Do you still want to "
                        "translate ?".format(path)
                    )
                msg.setText(text)
                if msg.exec_() != qt.QMessageBox.Ok:
                    return

            try:
                possible_entries = nxtomomill_converter.get_bliss_tomo_entries(
                    path, scan_titles=nxtomomill_converter.DEFAULT_SCAN_TITLES
                )
            except Exception as e:
                logger.error("Faild to find entries for {}".format(path))
                possible_entries = []

        for entry in possible_entries:
            scan = HDF5TomoScan(scan=path, entry=entry)
            scan_obj = _ScanList.add(self, scan)
            if scan_obj:
                self._addScanIDItem(str(scan_obj))


def main():
    app = qt.QApplication([])
    s = GenericDataListDialog()
    s.show()
    app.exec_()


if __name__ == "__main__":
    main()



aliases = {
    # define aliases for control
    "orangecontrib.tomwer.widgets.control.DataListOW.DataListenerOW.DataListenerOW": "tomwer.core.process.utils.IgnoreProcess",
    "orangecontrib.tomwer.widgets.control.DataListOW.DataListOW": "tomwer.core.process.utils.IgnoreProcess",
    "orangecontrib.tomwer.widgets.control.DataListOW.DataSelectorOW.DataSelectorOW": "tomwer.core.process.utils.IgnoreProcess",
    "orangecontrib.tomwer.widgets.control.DataListOW.DataValidatorOW.DataValidatorOW": "tomwer.core.process.utils.IgnoreProcess",
    "orangecontrib.tomwer.widgets.control.DataListOW.DataWatcherOW.DataWatcherOW": "tomwer.core.process.utils.IgnoreProcess",
    "orangecontrib.tomwer.widgets.control.DataListOW.FilterOW.FilterOW": "tomwer.core.process.utils.IgnoreProcess",
    "orangecontrib.tomwer.widgets.control.NXTomomillOW.NXTomomillOW": "tomwer.core.process.control.nxtomomill.NxTomomillProcess",
    # define aliases for other
    "orangecontrib.tomwer.widgets.other.PythonScriptOW.OWPythonScript": "tomwer.core.process.script.python.PythonScript",
    # define aliases for reconstruction
    "orangecontrib.tomwer.widgets.reconstruction.AxisOW.AxisOW": "tomwer.core.process.reconstruction.axis.AxisProcess",
    "orangecontrib.tomwer.widgets.reconstruction.DarkRefAndCopyOW.DarkRefAndCopyOW": "tomwer.core.process.reconstruction.darkref.darkrefs.DarkRefs",
    "orangecontrib.tomwer.widgets.reconstruction.NabuOW.NabuOW": "tomwer.core.process.reconstruction.nabu.nabuslices.NabuSlices",
    "orangecontrib.tomwer.widgets.reconstruction.NabuVolumeOW.NabuVolumeOW": "tomwer.core.process.reconstruction.nabu.nabuvolume.NabuVolume",
    "orangecontrib.tomwer.widgets.reconstruction.FtseriesOW.FtseriesOW": "tomwer.core.process.reconstruction.ftseries.ftseries._Ftseries",
    "orangecontrib.tomwer.widgets.reconstruction.TofuOW.TofuOW": "tomwer.core.process.reconstruction.lamino.tofu.LaminoReconstruction",
    # define aliases for visualization
    "orangecontrib.tomwer.widgets.visualization.ImageStackviewerOW.ImageStackviewerOW": "tomwer.core.process.utils.IgnoreProcess",
    "orangecontrib.tomwer.widgets.visualization.LivesliceOW.LivesliceOW": "tomwer.core.process.utils.IgnoreProcess",
    "orangecontrib.tomwer.widgets.visualization.RadioStackOW.RadioStackOW": "tomwer.core.process.utils.IgnoreProcess",
    "orangecontrib.tomwer.widgets.visualization.SampleMovedOW.SampleMovedOW": "tomwer.core.process.utils.IgnoreProcess",
    "orangecontrib.tomwer.widgets.visualization.SinogramViewerOW.SinogramViewerOW": "tomwer.core.process.utils.IgnoreProcess",
    "orangecontrib.tomwer.widgets.visualization.SliceStackOW.SliceStackOW": "tomwer.core.process.utils.IgnoreProcess",
    # define aliases for edit
    "orange.widgets.tomwer.edit.DarkFlatPatchOW.DarkFlatPatchOW": "tomwer.core.process.edit.darkflatpatch.DarkFlatPatch",
    "orange.widgets.tomwer.edit.DarkFlatPatchOW.ImageKeyEditorOW": "tomwer.core.process.edit.imagekeyeditor.ImageKeyEditor",
}


# aliases used to avoid executing orange widgets directly (and avoid Qt stuff...)
# NOTES:
#
# - dark ref copy is not managed by the process application. For now we don't have any process consistancy and we have to create each time a new process.
#
# Tested:
#
# - NXTomomill
# - Darkref
# - Axis
# - Nabu
# - PythonScript
#
# For now Several processes are just 'ignore' by the pipeline:
#



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
import sys
from silx.gui import qt
import argparse
from tomwer.gui import icons
from tomwer.gui.utils.splashscreen import getMainSplashScreen
from tomwer.core.scan.scanfactory import ScanFactory
from tomwer.core.scan.hdf5scan import HDF5TomoScan
from tomwer.gui.visualization.diffviewer import DiffFrameViewer
import signal

logging.basicConfig()
_logger = logging.getLogger(__name__)


def getinputinfo():
    return "tomwer nabu [scan_path]"


def sigintHandler(*args):
    """Handler for the SIGINT signal."""
    qt.QApplication.quit()


def main(argv):
    import os

    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument(
        "scan_path", help="Data file to show (h5 file, edf files, spec files)"
    )
    parser.add_argument(
        "--entry", default=None, help="An entry should be specify for hdf5 files"
    )
    options = parser.parse_args(argv[1:])

    app = qt.QApplication.instance() or qt.QApplication([])

    qt.QLocale.setDefault(qt.QLocale(qt.QLocale.English))
    qt.QLocale.setDefault(qt.QLocale.c())
    signal.signal(signal.SIGINT, sigintHandler)
    sys.excepthook = qt.exceptionHandler
    timer = qt.QTimer()
    timer.start(500)
    # Application have to wake up Python interpreter, else SIGINT is not
    # catch
    timer.timeout.connect(lambda: None)

    splash = getMainSplashScreen()
    options.scan_path = options.scan_path.rstrip(os.path.sep)
    if options.entry is None:
        scan = ScanFactory.create_scan_object(options.scan_path)
    else:
        scan = HDF5TomoScan(scan=options.scan_path, entry=options.entry)
    if scan is None:
        raise ValueError(
            "Given scan path is not recognized as a path" "containing a scan"
        )
    widget = DiffFrameViewer(parent=None)
    widget.setScan(scan)
    # for the application we run for the reconstruction to be finished
    # to give back hand to the user
    widget.setWindowTitle("Frame diff")
    widget.setWindowIcon(icons.getQIcon("tomwer"))
    splash.finish(widget)
    widget.show()
    app.exec_()


if __name__ == "__main__":
    main(sys.argv)



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/

__authors__ = ["H. Payno"]
__license__ = "MIT"
__date__ = "24/01/2017"


import logging
import numpy
import os
import shutil
import tempfile
import time
import h5py
import unittest
from glob import glob
from silx.gui.utils.testutils import TestCaseQt
from silx.gui import qt
from tomwer.test.utils import UtilsTest
from orangecontrib.tomwer.widgets.control.DataTransfertOW import DataTransfertOW
from tomwer.gui.qtapplicationmanager import QApplicationManager
from tomwer.core.scan.scanfactory import ScanFactory
from tomwer.core.utils.scanutils import MockEDF
from tomwer.core.scan.hdf5scan import HDF5TomoScan
from tomwer.synctools.rsyncmanager import RSyncManager

_qapp = QApplicationManager()

logging.disable(logging.INFO)


class TestEDFFolderTransfertWidget(TestCaseQt):
    """class testing the DataTransfertOW within an EDF acquisition"""

    def setUp(self):
        TestCaseQt.setUp(self)
        self.n_file = 10
        self.sourcedir = tempfile.mkdtemp()
        assert os.path.isdir(self.sourcedir)
        self.targettedir = tempfile.mkdtemp()
        assert os.path.isdir(self.targettedir)
        MockEDF.fastMockAcquisition(self.sourcedir, n_radio=self.n_file)
        self.scan = ScanFactory.create_scan_object(self.sourcedir)

        self.folderTransWidget = DataTransfertOW()
        self.folderTransWidget.turn_off_print = True
        self.folderTransWidget.setDestDir(self.targettedir)
        self.folderTransWidget._copying = False
        self.folderTransWidget.setForceSync(True)

    def tearDown(self):
        self.folderTransWidget.settingsHandler.removeCallback(
            self.folderTransWidget._updateSettingsVals
        )
        self.folderTransWidget.setAttribute(qt.Qt.WA_DeleteOnClose)
        self.folderTransWidget.close()
        self.folderTransWidget = None

        if os.path.isdir(self.sourcedir):
            shutil.rmtree(self.sourcedir)
        if os.path.isdir(self.targettedir):
            shutil.rmtree(self.targettedir)
        TestCaseQt.tearDown(self)

    def testMoveFiles(self):
        """
        simple test that files are correctly moved
        """
        self.folderTransWidget.process(self.scan, move=True, noRsync=True)

        outputdir = os.path.join(self.targettedir, os.path.basename(self.sourcedir))
        timeout = 1
        while (
            (os.path.isdir(self.sourcedir))
            and timeout > 0
            or self.folderTransWidget.isCopying()
        ):
            timeout = timeout - 0.1
            time.sleep(0.1)
            _qapp.processEvents()

        self.assertTrue(os.path.isdir(outputdir))
        self.assertTrue(self.checkDataCopied())

    def testCopyFiles(self):
        """
        Simple test that file are copy and deleted
        """
        self.folderTransWidget.process(self.scan, move=False, noRsync=True)

        timeout = 1
        outputdir = os.path.join(self.targettedir, os.path.basename(self.sourcedir))
        while (
            os.path.isdir(self.sourcedir) and timeout > 0
        ) or self.folderTransWidget.isCopying():
            timeout = timeout - 0.1
            time.sleep(0.1)
            _qapp.processEvents()

        time.sleep(1)

        self.assertTrue(os.path.isdir(outputdir))
        self.assertTrue(self.checkDataCopied())

    def checkDataCopied(self):
        outputdir = os.path.join(self.targettedir, os.path.basename(self.sourcedir))
        outputFiles = os.listdir(outputdir)
        inputFile = glob(self.sourcedir)

        return (
            (len(inputFile) == 0)
            and (len(outputFiles) == (self.n_file + 3))
            and (not os.path.isdir(self.sourcedir))
        )


def suite():
    test_suite = unittest.TestSuite()
    for ui in (TestEDFFolderTransfertWidget,):
        test_suite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(ui))
    return test_suite


if __name__ == "__main__":
    unittest.main(defaultTest="suite")



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/

__authors__ = ["H. Payno"]
__license__ = "MIT"
__date__ = "17/03/2020"


import logging
import unittest
import tempfile
import shutil
import time
import os
from silx.gui import qt
from tomwer.core.utils import ignoreLowMemory
from tomwer.core import settings
from orangecontrib.tomwer.widgets.control.DataListenerOW import DataListenerOW
from orangecontrib.tomwer.test.utils import OrangeWorflowTest
from tomwer.test.utils import UtilsTest
from tomwer.synctools.datalistener import _mock_acquisition_info
from silx.gui.utils.testutils import TestCaseQt
from silx.gui.utils.testutils import SignalListener
from tomwer.test.utils import skip_gui_test, skip_orange_workflows_test


logging.disable(logging.INFO)


@unittest.skip("Need re-work")
@unittest.skipIf(skip_gui_test(), reason="skip gui test")
class TestDataListener(TestCaseQt):
    """Test the DataListener Orange Widget"""

    def setUp(self):
        TestCaseQt.setUp(self)
        self._src_dir = tempfile.mkdtemp()
        nxtomomill_file = "tomo_v2_external.h5"
        nxtomomill_folder = os.path.join(self._src_dir, "tomo_v2")
        os.mkdir(nxtomomill_folder)
        self.nxtomomill_file = os.path.join(nxtomomill_folder, nxtomomill_file)
        shutil.copyfile(
            src=UtilsTest.getH5Dataset(folderID=nxtomomill_file),
            dst=self.nxtomomill_file,
        )

        self.widget = DataListenerOW(parent=None)
        scan_numbers = ["426.1", "427.1", "428.1", "429.1", "430.1", "431.1"]
        self.widget.setMock(
            mock=True,
            acquisitions=(
                _mock_acquisition_info(
                    master_file=self.nxtomomill_file,
                    entry="425.1",
                    waiting_time=0.1,
                    scan_numbers=scan_numbers,
                ),
            ),
        )
        self.nx_file = os.path.join(nxtomomill_folder, "tomo_v2_external_425_1.nx")
        assert not os.path.exists(self.nx_file)

    def tearDown(self):
        self.widget.setAttribute(qt.Qt.WA_DeleteOnClose)
        self.widget.close()
        del self.widget
        self.widget = None
        self.qapp.processEvents()
        shutil.rmtree(self._src_dir)
        TestCaseQt.tearDown(self)

    def test(self):
        self.widget.activate()
        self.widget.show()
        self.qapp.processEvents()
        # check observation is on going
        time.sleep(0.2)
        self.qapp.processEvents()
        self.assertEqual(self.widget._get_n_scan_observe(), 1)
        self.assertEqual(self.widget._get_n_scan_finished(), 0)

        self.qapp.processEvents()
        time.sleep(1.2)
        self.qapp.processEvents()
        # check acquisition is moved to 'finished'
        self.assertEqual(self.widget._get_n_scan_observe(), 0)
        self.assertEqual(self.widget._get_n_scan_finished(), 1)

        self.qapp.processEvents()
        time.sleep(0.2)
        self.qapp.processEvents()
        # check conversion has been made
        self.assertTrue(os.path.exists(self.nx_file))


@unittest.skipIf(skip_gui_test(), reason="skip gui test")
@unittest.skipIf(skip_orange_workflows_test(), reason="skip orange workflows tests")
class TestListenerToTransfert(OrangeWorflowTest):
    """class testing the following flow:

    - data listener
    - dark and flat field construction
    - nabu reconstruction
    - data validator
    - data transfert

    This test conversion from bliss .hdf5 to tomonx .nx file then a 'classical'
    workflow
    """

    @classmethod
    def setUpClass(cls):
        OrangeWorflowTest.setUpClass()
        # create widgets

        cls.dataListenerNode = cls.addWidget(
            cls, "orangecontrib.tomwer.widgets.control.DataListenerOW.DataListenerOW"
        )
        cls.dkrfNode = cls.addWidget(
            cls,
            "orangecontrib.tomwer.widgets.reconstruction.DarkRefAndCopyOW.DarkRefAndCopyOW",
        )
        cls.nabuNode = cls.addWidget(
            cls, "orangecontrib.tomwer.widgets.reconstruction.NabuOW.NabuOW"
        )
        cls.dataValidatorNode = cls.addWidget(
            cls, "orangecontrib.tomwer.widgets.control.DataValidatorOW.DataValidatorOW"
        )
        cls.dataTransfertNode = cls.addWidget(
            cls, "orangecontrib.tomwer.widgets.control.DataTransfertOW.DataTransfertOW"
        )

        # Let Orange process events (node creations)
        cls.processOrangeEvents(cls)

        # linking the workflow
        cls.link(cls, cls.dataListenerNode, "data", cls.dkrfNode, "data")
        cls.link(cls, cls.dkrfNode, "data", cls.nabuNode, "data")
        cls.link(cls, cls.nabuNode, "data", cls.dataValidatorNode, "data")
        cls.link(cls, cls.dataValidatorNode, "data", cls.dataTransfertNode, "data")
        cls.link(
            cls,
            cls.dataValidatorNode,
            "change recons params",
            cls.nabuNode,
            "change recons params",
        )

        # Let Orange process events (node creations)
        cls.processOrangeEvents(cls)

        # getting the widgets
        cls.dataListenerWidget = cls.getWidgetForNode(cls, cls.dataListenerNode)
        cls.dkrfWidget = cls.getWidgetForNode(cls, cls.dkrfNode)
        cls.nabuWidget = cls.getWidgetForNode(cls, cls.nabuNode)
        cls.dataValidatorWidget = cls.getWidgetForNode(cls, cls.dataValidatorNode)
        cls.dataTransfertWidget = cls.getWidgetForNode(cls, cls.dataTransfertNode)

        cls.processOrangeEvents(cls)

    def setUp(self):
        OrangeWorflowTest.setUp(self)
        # source directory
        self._src_dir = tempfile.mkdtemp()
        # destination dir
        self._dst_dir = tempfile.mkdtemp()
        settings._set_lbsram_path(self._src_dir)
        settings._set_dest_path(self._dst_dir)

        self._folder_tomo_v2 = os.path.join(self._src_dir, "tomo_v2")
        o_dataset = UtilsTest.getH5Dataset(folderID="tomo_v2")
        shutil.copytree(o_dataset, os.path.join(self._src_dir, "tomo_v2"))
        self.bliss_file = os.path.join(self._src_dir, "tomo_v2", "tomo_v2_external.h5")
        self.final_dest_dir = os.path.join(self._dst_dir, "tomo_v2")

        # warm widget that this is for CI
        self.dkrfWidget.setForceSync(True)
        self.nabuWidget.setDryRun(True)
        # define the acquisition to mock
        scan_numbers = ["426.1", "427.1", "428.1", "429.1", "430.1", "431.1"]
        self.dataListenerWidget.setMock(
            mock=True,
            acquisitions=(
                _mock_acquisition_info(
                    master_file=self.bliss_file,
                    entry="425.1",
                    waiting_time=0.1,
                    scan_numbers=scan_numbers,
                ),
            ),
        )
        self.nx_file = os.path.join(
            self._src_dir, "tomo_v2", "tomo_v2_external_425_1.nx"
        )

        # mock lbsram
        settings.mock_lsbram(True)
        settings._set_lbsram_path(self._src_dir)
        settings._set_dest_path(self._dst_dir)

        # create signal listener
        self._dataTransfertListener = SignalListener()
        self.dataTransfertWidget.scanready.connect(self._dataTransfertListener)
        self.dataTransfertWidget.setForceSync(True)
        ignoreLowMemory(True)

    @classmethod
    def tearDownClass(cls):
        for node in (
            cls.dataListenerNode,
            cls.dkrfNode,
            cls.nabuNode,
            cls.dataValidatorNode,
            cls.dataTransfertNode,
        ):
            cls.removeNode(cls, node)
        cls.dataListenerWidget = None
        cls.dkrfWidget = None
        cls.nabuWidget = None
        cls.dataValidatorWidget = None
        cls.dataTransfertWidget = None
        OrangeWorflowTest.tearDownClass()

    def tearDown(self):
        self.nabuWidget.settingsHandler.removeCallback(
            self.nabuWidget._updateSettingsVals
        )
        self.dataListenerWidget.activate(False)
        # reset default configuration
        ignoreLowMemory(False)
        if os.path.exists(self._src_dir):
            shutil.rmtree(self._src_dir)
        if os.path.exists(self._dst_dir):
            shutil.rmtree(self._dst_dir)
        settings._set_lbsram_path("/lbsram")
        settings._set_dest_path("")

        OrangeWorflowTest.tearDown(self)

    def testFlow(self):
        """
        Make sure all possibles input are managed and processed from one
        widget to the others
        """
        self.assertFalse(os.path.exists(self.final_dest_dir))
        self.process_datalistener()

        # check that synchronization has been done
        self.assertTrue(os.path.exists(self.final_dest_dir))
        self.wait_and_process_datavalidator()
        # make sure everything has been processed
        for i in range(20):
            time.sleep(0.02)
            self.qapp.processEvents()

        # validate the current scan
        self.dataValidatorWidget.validateCurrentScan()
        self.check_data_transfert()

    def process_datalistener(self):
        self.dataListenerWidget.activate()
        self.dataListenerWidget.show()
        self.qapp.processEvents()
        # check observation is on going
        time.sleep(0.2)
        self.qapp.processEvents()
        self.assertEqual(self.dataListenerWidget._get_n_scan_observe(), 1)
        self.assertEqual(self.dataListenerWidget._get_n_scan_finished(), 0)

        self.qapp.processEvents()
        time.sleep(1.2)
        self.qapp.processEvents()
        # check acquisition is moved to 'finished'
        self.assertEqual(self.dataListenerWidget._get_n_scan_observe(), 0)
        self.assertEqual(self.dataListenerWidget._get_n_scan_finished(), 1)

        self.qapp.processEvents()
        time.sleep(0.2)
        self.qapp.processEvents()
        # check conversion has been made
        self.assertTrue(os.path.exists(self.nx_file))

    def wait_and_process_datavalidator(self):
        def wait():
            timeout = 4
            while len(self.dataValidatorWidget._scansToValidate) and timeout >= 0:
                time.sleep(0.05)
                timeout -= 0.05

            if timeout < 0:
                raise TimeoutError("data validator never received a scan")

        wait()
        self.dataValidatorWidget.show()

    def check_data_transfert(self):
        timeout = 20
        while (self._dataTransfertListener.callCount() == 0) and timeout >= 0:
            time.sleep(0.05)
            timeout -= 0.05
            self.qapp.processEvents()

        if timeout < 0:
            raise TimeoutError("Data transfert was never processed")


def suite():
    test_suite = unittest.TestSuite()
    for ui in (TestListenerToTransfert, TestDataListener):
        test_suite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(ui))
    return test_suite


if __name__ == "__main__":
    unittest.main(defaultTest="suite")



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/

__authors__ = ["H. Payno"]
__license__ = "MIT"
__date__ = "22/03/2018"


import tempfile
import logging
import unittest
import shutil
import time
from orangecontrib.tomwer.test.utils import OrangeWorflowTest
from tomwer.test.utils import UtilsTest
from tomwer.test.utils import skip_gui_test, skip_orange_workflows_test
from tomwer.core.utils.scanutils import MockHDF5


logging.disable(logging.INFO)


@unittest.skipIf(skip_orange_workflows_test(), reason="skip orange workflows tests")
@unittest.skipIf(skip_gui_test(), reason="skip gui test")
class TestScanListSampleMovedWorkflow(OrangeWorflowTest):
    """Make sure the sample moved is correctly connecting to the orange-canvas
    and that it will display requested scans
    """

    def setUp(self):
        super(TestScanListSampleMovedWorkflow, self).setUp()
        dataset = "D2_H2_T2_h_"
        self.EDFdataTestDir = UtilsTest.getEDFDataset(dataset)

        self.output_folder = tempfile.mkdtemp()

        self._hdf5_n_proj = 10
        self._hdf5_n_alignment = 2
        hdf5_mock = MockHDF5(
            scan_path=self.output_folder,
            n_ini_proj=self._hdf5_n_proj,
            n_proj=self._hdf5_n_proj,
            scan_range=180,
        )
        hdf5_mock.add_alignment_radio(index=self._hdf5_n_alignment, angle=90)
        hdf5_mock.add_alignment_radio(index=self._hdf5_n_alignment + 1, angle=0)
        self.hdf5_scan = hdf5_mock.scan

        self.sampleMovedWidget.clear()
        self.scanListWidget.clear()
        self.qapp.processEvents()

    def tearDow(self):
        shutil.rmtree(self.output_folder)
        super(TestScanListSampleMovedWorkflow, self).tearDow()

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        # create widgets
        cls.nodeScanList = cls.addWidget(
            cls, "orangecontrib.tomwer.widgets.control.DataListOW.DataListOW"
        )
        cls.nodeSampleMoved = cls.addWidget(
            cls,
            "orangecontrib.tomwer.widgets.visualization.SampleMovedOW.SampleMovedOW",
        )
        cls.processOrangeEvents(cls)

        cls.link(cls, cls.nodeScanList, "data", cls.nodeSampleMoved, "data")
        cls.processOrangeEvents(cls)

        cls.scanListWidget = cls.getWidgetForNode(cls, cls.nodeScanList)
        cls.sampleMovedWidget = cls.getWidgetForNode(cls, cls.nodeSampleMoved)

    @classmethod
    def tearDownClass(cls):
        for node in (cls.nodeScanList, cls.nodeSampleMoved):
            cls.removeNode(cls, node)
        cls.scanListWidget = None
        cls.sampleMovedWidget = None
        super().tearDownClass()


@unittest.skipIf(skip_gui_test(), reason="skip gui test")
class TestScanListSampleMovedWorkflowEDF(TestScanListSampleMovedWorkflow):
    def testEDF(self):
        self.assertEqual(len(self.sampleMovedWidget._mainWidget._images), 0)

        self.scanListWidget.add(self.EDFdataTestDir)
        self.scanListWidget._sendList()
        for i in range(20):
            time.sleep(0.05)
            self.qapp.processEvents()
        self.assertEqual(len(self.sampleMovedWidget._mainWidget._images), 3605)


@unittest.skipIf(skip_gui_test(), reason="skip gui test")
class TestScanListSampleMovedWorkflowHDF5(TestScanListSampleMovedWorkflow):
    def testHDF5(self):
        self.assertEqual(len(self.sampleMovedWidget._mainWidget._images), 0)
        self.assertTrue(self.sampleMovedWidget._scanNameQLabel.text() == "")

        self.scanListWidget.add(self.hdf5_scan)
        self.scanListWidget._sendList()
        for i in range(20):
            time.sleep(0.05)
            self.qapp.processEvents()
        n_total_proj = self._hdf5_n_proj + self._hdf5_n_alignment
        self.assertEqual(len(self.sampleMovedWidget._mainWidget._images), n_total_proj)


def suite():
    test_suite = unittest.TestSuite()
    for ui in (
        TestScanListSampleMovedWorkflow,
        TestScanListSampleMovedWorkflowEDF,
        TestScanListSampleMovedWorkflowHDF5,
    ):
        test_suite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(ui))
    return test_suite


if __name__ == "__main__":
    unittest.main(defaultTest="suite")



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
import sys
import argparse
from pypushflow.representation.scheme.ows_parser import OwsParser
from pypushflow import Workflow
import tomwer.version
import pypushflow.version
import os

logging.basicConfig(level=logging.DEBUG)
_logger = logging.getLogger(__name__)


def _convert(scheme, output_file, overwrite):
    """

    :param scheme:
    :param scan:
    :param timeout:
    :return:
    """
    _logger.warning("translate {} to {}".format(scheme, output_file))

    if os.path.exists(output_file):
        if overwrite is True:
            os.remove(output_file)
        else:
            raise ValueError("{} already exists.".format(output_file))

    with open(output_file, "w+") as file_:
        file_.write(_dump_info_generation())

    workflow = Workflow.ProcessableWorkflow(scheme)
    converter = Workflow.Converter(workflow=workflow, output_file=output_file)
    converter.process()

    # set up workflow
    with open(output_file, mode="a") as file_:
        file_.write(_dump_executable_script_section())
    _logger.info(
        "translation finished. You can execute python {} [scan path] [[--entry]]".format(
            output_file
        )
    )


def _dump_info_generation():
    return (
        "# This file has been generated automatically using \n"
        "# pypushflow {} and tomwer {}\n".format(
            pypushflow.version.version, tomwer.version.version
        )
    )


def _dump_executable_script_section():
    return """
from tomwer.core.process.utils import IgnoreProcess

if __name__ == '__main__':
    import argparse
    import sys
    from tomwer.core.scan.scanfactory import ScanFactory
    from tomwer.core.scan.hdf5scan import HDF5TomoScan

    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument(
        'scan_path',
        help='Data file to show (h5 file, edf files, spec files)')
    parser.add_argument(
        '--entry',
        default=None,
        help='An entry should be specify for hdf5 files')

    options = parser.parse_args(sys.argv[1:])
    if options.entry is None:
        scan = ScanFactory.create_scan_object(options.scan_path)
    else:
        scan = ScanFactory.create_scan_object(options.scan_path,
                                              entry=options.entry,
                                              accept_bliss_scan=True)
    if scan is None:
        raise ValueError('Given scan path is not recognized as a path'
                         'containing a scan')
    main(input_data=scan, channel='data')
    """


def main(argv):
    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument(
        "workflow_file",
        help="Path to the .ows file defining the workflow to process with the"
        "provided scan",
    )
    parser.add_argument("output_file", help="Output python file")
    parser.add_argument(
        "--overwrite",
        help="Overwrite output file if exists",
        default=False,
        action="store_true",
    )
    parser.add_argument(
        "--debug",
        dest="debug",
        action="store_true",
        default=False,
        help="Set logging system in debug mode",
    )
    options = parser.parse_args(argv[1:])
    if not options.output_file.lower().endswith(".py"):
        options.output_file = options.output_file + ".py"
    # tune the log level
    log_level = logging.INFO
    if options.debug is True:
        log_level = logging.DEBUG

    for log_ in ("tomwer", "pypushflow"):
        logging.getLogger(log_).setLevel(log_level)

    scheme = OwsParser.scheme_load(options.workflow_file, load_handlers=True)
    _convert(
        scheme=scheme, output_file=options.output_file, overwrite=options.overwrite
    )


if __name__ == "__main__":
    main(sys.argv)
# convert an ows file to a script calling tomwer low level processes.



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/
__authors__ = ["H. Payno"]
__license__ = "MIT"
__date__ = "09/02/2018"

import logging
import os
import shutil
import tempfile
import time
import unittest
from glob import glob

from orangecontrib.tomwer.test.utils import OrangeWorflowTest
from tomwer.core import utils
from tomwer.core.process.reconstruction.darkref.darkrefs import DarkRefs
from tomwer.core.settings import mock_lsbram
from tomwer.test.utils import UtilsTest
from tomwer.test.utils import skip_gui_test, skip_orange_workflows_test
from silx.gui import qt
from silx.gui.utils.testutils import SignalListener
from tomwer.core.scan.scanfactory import ScanFactory

logging.disable(logging.INFO)


@unittest.skipIf(skip_orange_workflows_test(), reason="skip orange workflows tests")
@unittest.skipIf(skip_gui_test(), reason="skip gui test")
class TestConstructionDarkAndFlatField(OrangeWorflowTest):
    """
    test the workflow composed of the following widgets :
        - DataWatcherOW
        - DarkRefsCopy : Make sure the refCopy is correctly make
    """

    TIMEOUT = 20

    @classmethod
    def setUpClass(cls):
        OrangeWorflowTest.setUpClass()
        # create widgets
        cls.nodeDataWatcher = cls.addWidget(
            cls, "orangecontrib.tomwer.widgets.control.DataWatcherOW.DataWatcherOW"
        )
        cls.nodeDarkRefs = cls.addWidget(
            cls,
            "orangecontrib.tomwer.widgets.reconstruction.DarkRefAndCopyOW.DarkRefAndCopyOW",
        )
        cls.darkRefListener = SignalListener()
        cls.datawatcherListener = SignalListener()
        # Let Orange process events (node creations)
        cls.processOrangeEvents(cls)

        # linking the workflow
        cls.link(cls, cls.nodeDataWatcher, "data", cls.nodeDarkRefs, "data")

        # Let Orange process events (node creations)
        cls.processOrangeEvents(cls)

        # getting the widgets
        cls.datawatcherWidget = cls.getWidgetForNode(cls, cls.nodeDataWatcher)
        cls.darkRefsWidget = cls.getWidgetForNode(cls, cls.nodeDarkRefs)

        # set mock mode for FTSerieWidget
        # set data watcher ready for observation
        cls.datawatcherWidget.displayAdvancement = False
        # force Dark ref to be sync
        cls.darkRefsWidget.setForceSync(True)
        cls.darkRefsWidget.widget.sigScanReady.connect(cls.darkRefListener)
        cls.datawatcherWidget._widget.sigScanReady.connect(cls.datawatcherListener)

    @classmethod
    def tearDownClass(cls):
        for node in (cls.nodeDataWatcher, cls.nodeDarkRefs):
            cls.removeNode(cls, node)
        cls.app.processEvents()
        del cls.datawatcherWidget
        del cls.darkRefsWidget
        OrangeWorflowTest.tearDownClass()

    def setUp(self):
        OrangeWorflowTest.setUp(self)

        def prepareRefFolder():
            datasetID = "test10"
            self.inputFolder = tempfile.mkdtemp()
            self.scanFolder = os.path.join(self.inputFolder, datasetID)
            self.copytree = shutil.copytree(
                src=UtilsTest().getEDFDataset(datasetID), dst=self.scanFolder
            )

            [
                os.remove(f)
                for f in DarkRefs.getRefHSTFiles(self.scanFolder, prefix="refHST")
            ]
            [
                os.remove(f)
                for f in DarkRefs.getDarkHSTFiles(self.scanFolder, prefix="darkHST")
            ]

        prepareRefFolder()

        def prepareScanToProcess():
            self._refParentFolder = tempfile.mkdtemp()
            datasetRef = "test01"
            self.refFolder = os.path.join(self._refParentFolder, datasetRef)
            shutil.copytree(
                src=UtilsTest().getEDFDataset(datasetRef), dst=self.refFolder
            )
            # copy .info file to have coherent REf_N values
            shutil.copyfile(
                src=os.path.join(self._refParentFolder, "test01", "test01.info"),
                dst=os.path.join(self.scanFolder, "test10.info"),
            )
            assert len(glob(os.path.join(self.refFolder, "refHST*"))) > 0
            # dark file should be dark.edf and not darkHST.edf
            dark_HST = os.path.join(self.refFolder, "darkHST0000.edf")
            if os.path.exists(dark_HST):
                os.rename(dark_HST, os.path.join(self.refFolder, "dark.edf"))
            assert len(glob(os.path.join(self.refFolder, "dark.edf"))) == 1

        prepareScanToProcess()

        self.datawatcherWidget.setFolderObserved(self.inputFolder)
        self.darkRefsWidget.recons_params._set_skip_if_exist(True)
        self.darkRefsWidget.setRefsFromScan(
            ScanFactory.create_scan_object(self.refFolder)
        )
        self.darkRefsWidget.setModeAuto(False)
        utils.mockLowMemory(False)
        mock_lsbram(True)

        # reset listeners
        self.darkRefListener.clear()
        self.datawatcherListener.clear()

    def tearDown(self):
        self.datawatcherWidget._widget.stop()
        self.datawatcherWidget.setAttribute(qt.Qt.WA_DeleteOnClose)
        self.datawatcherWidget.close()
        self.datawatcherWidget = None
        shutil.rmtree(self._refParentFolder)
        shutil.rmtree(self.inputFolder)
        OrangeWorflowTest.tearDown(self)

    def testWorkflow(self):
        assert not self.datawatcherWidget._widget.isObserving
        self.datawatcherWidget.startObservation()
        timeout = TestConstructionDarkAndFlatField.TIMEOUT
        # wait until find the scan
        while self.datawatcherListener.callCount() < 1 and timeout > 0:
            self.app.processEvents()
            timeout -= 0.2
            time.sleep(0.2)

        if timeout < 0:
            raise TimeoutError("data watcher can't find any scan")

        # let dark ref process time to end
        timeout = TestConstructionDarkAndFlatField.TIMEOUT
        while timeout > 0 and self.darkRefListener.callCount() < 1:
            self.app.processEvents()
            timeout -= 0.2
            time.sleep(0.2)

        self.app.processEvents()
        if timeout < 0:
            raise TimeoutError("darkef didn't process")

        self.assertTrue(
            len(DarkRefs.getDarkHSTFiles(self.scanFolder, prefix="refHST")) > 0
        )
        self.assertTrue(
            len(DarkRefs.getRefHSTFiles(self.scanFolder, prefix="dark.edf")) > 0
        )
        while self.app.hasPendingEvents():
            self.app.processEvents()

        self.assertTrue(self.darkRefListener.callCount() == 1)
        self.assertTrue(self.datawatcherListener.callCount() == 1)


def suite():
    test_suite = unittest.TestSuite()
    for ui in (TestConstructionDarkAndFlatField,):
        test_suite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(ui))
    return test_suite


if __name__ == "__main__":
    unittest.main(defaultTest="suite")



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/
__authors__ = ["H. Payno"]
__license__ = "MIT"
__date__ = "09/02/2018"

import logging
import os
import shutil
import tempfile
import time
import unittest

from orangecontrib.tomwer.test.utils import OrangeWorflowTest
from tomwer.core.process.reconstruction.lamino.tofu import has_tofu
from tomwer.gui.qtapplicationmanager import QApplicationManager
from tomwer.test.utils import UtilsTest
from tomwer.test.utils import skip_gui_test, skip_orange_workflows_test

logging.disable(logging.INFO)

app = QApplicationManager()


@unittest.skipIf(skip_orange_workflows_test(), reason="skip orange workflows tests")
@unittest.skipIf(skip_gui_test(), reason="skip gui test")
@unittest.skipIf(has_tofu() is False, "Tofu missing")
class TestLaminoWidget(OrangeWorflowTest):
    """
    test the workflow composed of the following widgets :
        - Datawatcher
        - Lamino widget
    """

    @classmethod
    def setUpClass(cls):
        OrangeWorflowTest.setUpClass()
        # create widgets
        cls.nodeDataWatcher = cls.addWidget(
            cls, "orangecontrib.tomwer.widgets.control.DataWatcherOW.DataWatcherOW"
        )
        cls.nodeTofu = cls.addWidget(
            cls, "orangecontrib.tomwer.widgets.reconstruction.TofuOW.TofuOW"
        )

        # Let Orange process events (node creations)
        cls.processOrangeEvents(cls)

        # linking the workflow
        cls.link(cls, cls.nodeDataWatcher, "data", cls.nodeTofu, "data")

        # Let Orange process events (node creations)
        cls.processOrangeEvents(cls)

        # getting the widgets
        cls.dataWatcherWidget = cls.getWidgetForNode(cls, cls.nodeDataWatcher)
        cls.tofuWidget = cls.getWidgetForNode(cls, cls.nodeTofu)

    @classmethod
    def tearDownClass(cls):
        cls.dataWatcherWidget.close()
        for node in (cls.nodeDataWatcher, cls.nodeTofu):
            cls.removeNode(cls, node)
        del cls.dataWatcherWidget
        del cls.tofuWidget
        OrangeWorflowTest.tearDownClass()

    def setUp(self):
        OrangeWorflowTest.setUp(self)
        self.folder = tempfile.mkdtemp()
        datasetID = "test10"
        self.folder = os.path.join(self.folder, datasetID)
        shutil.copytree(src=UtilsTest().getEDFDataset(datasetID), dst=self.folder)

        self.dataWatcherWidget.setFolderObserved(self.folder)

    def tearDown(self):
        shutil.rmtree(self.folder)
        OrangeWorflowTest.tearDown(self)

    def testWorkflow(self):
        self.dataWatcherWidget._widget.mockObservation(self.folder)

        # let dark ref process time to end
        for _t in (1, 1):
            app.processEvents()
            time.sleep(_t)
            app.processEvents()
        self.assertTrue(
            self.tofuWidget._mainWidget._mainWidget._tabs._inputWidget.getPixelSize()
            == 3.02e-06
        )


def suite():
    test_suite = unittest.TestSuite()
    for ui in (TestLaminoWidget,):
        test_suite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(ui))
    return test_suite


if __name__ == "__main__":
    unittest.main(defaultTest="suite")



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/
__authors__ = ["H. Payno"]
__license__ = "MIT"
__date__ = "28/01/2019"

import logging
import unittest

from orangecontrib.tomwer.test.utils import OrangeWorflowTest
from tomwer.gui.qtapplicationmanager import QApplicationManager
from tomwer.test.utils import skip_gui_test

logging.disable(logging.INFO)

app = QApplicationManager()


@unittest.skipIf(skip_gui_test(), reason="skip gui test")
class TestDumpWidgets(OrangeWorflowTest):
    """
    simply create orange widgets in the orange canvas to make instances
    of those are made correctly
    """

    def testTimer(self):
        timer_node = self.addWidget(
            "orangecontrib.tomwer.widgets.control.TimerOW.TimerOW"
        )
        self.processOrangeEventsStack()
        timer_widget = self.getWidgetForNode(timer_node)
        self.assertTrue(timer_widget is not None)


def suite():
    test_suite = unittest.TestSuite()
    for ui in (TestDumpWidgets,):
        test_suite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(ui))
    return test_suite


if __name__ == "__main__":
    unittest.main(defaultTest="suite")



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/

__authors__ = ["H. Payno"]
__license__ = "MIT"
__date__ = "13/08/2020"

import logging
import unittest
from tomwer.test.utils import skip_gui_test
from orangecontrib.tomwer.test.utils import OrangeWorflowTest

logging.disable(logging.INFO)


@unittest.skipIf(skip_gui_test(), reason="skip gui test")
class TestVolumeWidget(OrangeWorflowTest):
    """Insure connection of the following widgets:

    - NabuOW
    - NabuVolumeOW
    - VolumeViewerOW
    """

    @classmethod
    def setUpClass(cls):
        OrangeWorflowTest.setUpClass()
        # create widgets
        cls.nabuSlice = cls.addWidget(
            cls, "orangecontrib.tomwer.widgets.reconstruction.NabuOW.NabuOW"
        )
        cls.nabuVolume = cls.addWidget(
            cls, "orangecontrib.tomwer.widgets.reconstruction.NabuVolumeOW.NabuVolumeOW"
        )
        cls.volumeViewer = cls.addWidget(
            cls,
            "orangecontrib.tomwer.widgets.visualization.VolumeViewerOW.VolumeViewerOW",
        )

        # Let Orange process events (node creations)
        cls.processOrangeEvents(cls)

        # linking the workflow
        cls.link(cls, cls.nabuSlice, "data", cls.nabuVolume, "data")
        cls.link(cls, cls.nabuVolume, "data", cls.volumeViewer, "data")

        # Let Orange process events (node creations)
        cls.processOrangeEvents(cls)

        # getting the widgets
        cls.nabuSliceWidget = cls.getWidgetForNode(cls, cls.nabuSlice)
        cls.nabuVolumeWidget = cls.getWidgetForNode(cls, cls.nabuVolume)
        cls.volumeViewerWidget = cls.getWidgetForNode(cls, cls.volumeViewer)

    @classmethod
    def tearDownClass(cls):
        for node in (cls.nabuSlice, cls.nabuVolume, cls.volumeViewer):
            cls.removeNode(cls, node)
        del cls.nabuSlice
        del cls.nabuVolume
        del cls.volumeViewer
        OrangeWorflowTest.tearDownClass()

    def testWorkflow(self):
        """Test construction is possible (in fact test is on the setUpClass
        part for now)"""
        pass


def suite():
    test_suite = unittest.TestSuite()
    for ui in (TestVolumeWidget,):
        test_suite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(ui))
    return test_suite


if __name__ == "__main__":
    unittest.main(defaultTest="suite")



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
import sys
from silx.gui import qt
import argparse
from tomwer.gui import icons
from tomwer.gui.utils.splashscreen import getMainSplashScreen
from tomwer.gui.reconstruction.nabu.slices import NabuDialog
from tomwer.core.scan.scanfactory import ScanFactory
from tomwer.core.scan.hdf5scan import HDF5TomoScan
from tomwer.core.process.reconstruction.nabu.nabuslices import run_slices_reconstruction
from tomwer.gui.visualization.dataviewer import DataViewer as _DataViewer
import signal

logging.basicConfig()
_logger = logging.getLogger(__name__)


def getinputinfo():
    return "tomwer nabu [scan_path]"


def sigintHandler(*args):
    """Handler for the SIGINT signal."""
    qt.QApplication.quit()


class DataViewer(_DataViewer):
    """The data viewer with an update when F5 is press"""

    closed = qt.Signal()
    """signal emitted when the widget is closing"""

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.setAttribute(qt.Qt.WA_DeleteOnClose)

    def keyPressEvent(self, event):
        if event.key() is qt.Qt.Key_F5:
            self._updateDisplay()

    def event(self, event) -> bool:
        if event.type() == qt.QEvent.DeferredDelete:
            self.cleanBeforeQuit()
            self.closed.emit()
        return super().event(event)


class NabuProcessingThread(qt.QThread):
    def init(self, scan, config, dry_run=False):
        self._scan = scan
        self._config = config
        self._dry_run = dry_run

    def run(self):
        run_slices_reconstruction(
            scan=self._scan, config=self._config, dry_run=self._dry_run
        )


class _CORWidget(qt.QWidget):
    def __init__(self, parent=None):
        qt.QWidget.__init__(self, parent)
        self.setLayout(qt.QHBoxLayout())
        self.layout().addWidget(qt.QLabel("center of rotation", self))
        self._corCN = qt.QCheckBox("auto", self)
        self.layout().addWidget(self._corCN)
        self._corQLE = qt.QLineEdit("0", self)
        self._corQLE.setValidator(qt.QDoubleValidator(self))
        self.layout().addWidget(self._corQLE)
        self._corQLE.setToolTip("Value should be in -width/2, width/2")

        # set up
        self._corCN.setChecked(True)
        self._corQLE.setEnabled(False)

        # connect signal / slot
        self._corCN.toggled.connect(self._corQLE.setDisabled)

    def getCOR(self):
        if self._corCN.isChecked():
            return "auto"
        else:
            return float(self._corQLE.text())


class NabuWindow(NabuDialog):
    def __init__(self, parent=None):
        NabuDialog.__init__(self, parent)
        self.setWindowFlags(qt.Qt.Widget)
        self.setAttribute(qt.Qt.WA_DeleteOnClose)

        self.__scan = None
        self.__processingThread = NabuProcessingThread()
        self._dry_run = False

        # add center of rotation widget
        self._corWidget = _CORWidget(parent=self)
        rw = self._widget._mainWidget._configuration._reconstructionWidget
        rw.layout().addWidget(self._corWidget, 99, 0, 1, 2)

        self._dataViewer = None
        self.setModal(False)

        # add view button
        self._showPB = qt.QPushButton("show scan", self)
        self._buttons.addButton(self._showPB, qt.QDialogButtonBox.ActionRole)

        # connect signal / slot
        self._computePB.released.connect(self._requireComputation)
        self._showPB.released.connect(self._showScan)

        # export API
        self.getCOR = self._corWidget.getCOR

    def _showScan(self):
        self._getDataViewer().setScan(self.__scan)
        self._getDataViewer().show()
        self._getDataViewer().raise_()

    def _getDataViewer(self):
        if self._dataViewer is None:
            self._dataViewer = DataViewer(parent=None)
            self._dataViewer.closed.connect(self._viewerDeleted)

        return self._dataViewer

    def _viewerDeleted(self):
        self._dataViewer = None

    def setScan(self, scan):
        self.__scan = scan
        NabuDialog.setScan(self, scan)

    def getScan(self):
        return self.__scan

    def getConfiguration(self):
        config = super().getConfiguration()
        cor = self.getCOR()
        if cor != "auto":
            # move the cor value to the nabu reference
            cor = cor + self.__scan.dim_1 // 2
        config["reconstruction"]["rotation_axis_position"] = str(cor)
        return config

    def _requireComputation(self, *arg, **kwargs):
        self.setEnabled(False)
        qt.QApplication.setOverrideCursor(qt.Qt.WaitCursor)
        scan = self.getScan()
        if scan is not None:
            self._launchComputation(scan=scan, config=self.getConfiguration())

    def _computationEnded(self):
        self.setEnabled(True)
        qt.QApplication.restoreOverrideCursor()
        if self._dataViewer is not None:
            self._dataViewer.setScan(self.__scan)

    def getProcessingThread(self):
        return self.__processingThread

    def _launchComputation(self, scan, config):
        # update the processing thread
        thread = self.getProcessingThread()
        thread.init(scan=scan, config=config, dry_run=self._dry_run)
        thread.finished.connect(self._computationEnded)

        # start processing
        thread.start(priority=qt.QThread.LowPriority)

    def set_dry_run(self, dry_run):
        self._dry_run = dry_run

    def close(self):
        super().close()
        if self._dataViewer is not None:
            self._dataViewer.close()
            self._dataViewer = None

    def _aboutToQuit(self):
        if self._dataViewer is not None:
            self._dataViewer.cleanBeforeQuit()


def main(argv):
    import os

    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument(
        "scan_path", help="Data file to show (h5 file, edf files, spec files)"
    )
    parser.add_argument(
        "--entry", default=None, help="An entry should be specify for hdf5 files"
    )
    parser.add_argument(
        "--output-dir", help="output directory of the reconstruction(s)"
    )
    parser.add_argument(
        "--debug",
        dest="debug",
        action="store_true",
        default=False,
        help="Set logging system in debug mode",
    )
    parser.add_argument(
        "--dry-run",
        dest="dry_run",
        action="store_true",
        default=False,
        help="Only create the .nabu file and print the configuration used",
    )
    options = parser.parse_args(argv[1:])

    app = qt.QApplication.instance() or qt.QApplication([])

    qt.QLocale.setDefault(qt.QLocale(qt.QLocale.English))
    qt.QLocale.setDefault(qt.QLocale.c())
    signal.signal(signal.SIGINT, sigintHandler)
    sys.excepthook = qt.exceptionHandler
    timer = qt.QTimer()
    timer.start(500)
    # Application have to wake up Python interpreter, else SIGINT is not
    # catch
    timer.timeout.connect(lambda: None)

    splash = getMainSplashScreen()
    options.scan_path = options.scan_path.rstrip(os.path.sep)
    if options.entry is None:
        scan = ScanFactory.create_scan_object(options.scan_path)
    else:
        scan = HDF5TomoScan(scan=options.scan_path, entry=options.entry)
    if scan is None:
        raise ValueError(
            "Given scan path is not recognized as a path" "containing a scan"
        )
    widget = NabuWindow(parent=None)
    widget.setScan(scan)
    scan.set_process_index_frm_tomwer_process_file()
    if options.output_dir is not None:
        widget.setOutputDir(options.output_dir)
    # for the application we run for the reconstruction to be finished
    # to give back hand to the user
    widget.set_dry_run(options.dry_run)
    widget.setWindowTitle("Nabu reconstruction")
    widget.setWindowIcon(icons.getQIcon("tomwer"))
    splash.finish(widget)
    widget.show()
    qt.QApplication.instance().aboutToQuit.connect(widget._aboutToQuit)

    app.exec_()


if __name__ == "__main__":
    main(sys.argv)



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/
__authors__ = ["H. Payno"]
__license__ = "MIT"
__date__ = "24/01/2017"

import copy
import logging
import os
import shutil
import tempfile
import unittest
import time
from silx.gui.utils.testutils import SignalListener

from orangecontrib.tomwer.test.utils import OrangeWorflowTest
from tomwer.core.process.reconstruction.ftseries.params.fastsetupdefineglobals import (
    FastSetupAll,
)
from tomwer.synctools.stacks.reconstruction.ftseries import _ReconsFtSeriesThread
from tomwer.test.utils import UtilsTest
from tomwer.test.utils import skip_gui_test, skip_orange_workflows_test
from tomwer.core.utils.scanutils import MockEDF

logging.disable(logging.INFO)


@unittest.skipIf(skip_orange_workflows_test(), reason="skip orange workflows tests")
@unittest.skipIf(skip_gui_test(), reason="skip gui test")
class TestScanListFTSerieWorkflow(OrangeWorflowTest):
    """Make sure the reconstruction of the second scan is executed with the
    correct reconstruction parameters.
    Set up is as following :
        - ScanList contains two datasets. Those scan can contains or not some .h5
        - FTSerie is activated with or without H5Exploration option
    """

    def setUp(self):
        super(TestScanListFTSerieWorkflow, self).setUp()
        self.inputDir = tempfile.mkdtemp()

        # define reconstruction parameters
        ft = FastSetupAll()
        self.default = copy.deepcopy(ft.structures)
        assert "FT" in ft.structures
        assert "NUM_PART" in ft.structures["FT"]
        assert ft.structures["FT"]["NUM_PART"] not in (1, 2)
        ft.structures["FT"]["NUM_PART"] = 1
        self.st1 = copy.deepcopy(ft.structures)
        ft.structures["FT"]["NUM_PART"] = 2
        self.st2 = copy.deepcopy(ft.structures)

    def tearDow(self):
        if os.path.isdir(self.inputDir):
            shutil.rmtree(self.inputDir)
        super(TestScanListFTSerieWorkflow, self).tearDow()

    @classmethod
    def setUpClass(cls):
        OrangeWorflowTest.setUpClass()
        # create widgets
        cls.nodeScanList = cls.addWidget(
            cls, "orangecontrib.tomwer.widgets.control.DataListOW.DataListOW"
        )
        cls.nodeFTSerie = cls.addWidget(
            cls, "orangecontrib.tomwer.widgets.control.FtseriesOW.FtseriesOW"
        )
        cls.processOrangeEvents(cls)

        cls.link(cls, cls.nodeScanList, "data", cls.nodeFTSerie, "data")
        cls.processOrangeEvents(cls)

        cls.scanListWidget = cls.getWidgetForNode(cls, cls.nodeScanList)
        cls.ftserieWidget = cls.getWidgetForNode(cls, cls.nodeFTSerie)
        _ReconsFtSeriesThread.setCopyH5FileReconsIntoFolder(True)

        # Set we only want to simulate the reconstruction
        cls.ftserieWidget._ftserie.reconsStack.setMockMode(True)

    @classmethod
    def tearDownClass(cls):
        for node in (cls.nodeScanList, cls.nodeFTSerie):
            cls.removeNode(cls, node)
        del cls.scanListWidget
        del cls.ftserieWidget
        OrangeWorflowTest.tearDownClass()

    def initData(self, data01H5, data10H5):
        """Init the two datasets and set the given .h5 file if any given
        The order of scanning is always : first dataset01, second dataset10

        :param data01H5: the values of the structures to save as a h5 file.
            Apply on the daset01.
            If None given then no H5 file will be saved
        :param data10H5: the values of the structures to save as a h5 file.
            Apply on the daset10.
            If None given then no H5 file will be saved
        """

        dataDir01 = UtilsTest.getEDFDataset("test01")
        dataDir10 = UtilsTest.getEDFDataset("test10")
        # removing any previous scan
        self.clearInputFolder()

        # set new scan
        self.dest01 = os.path.join(self.inputDir, os.path.basename(dataDir01))
        shutil.copytree(src=dataDir01, dst=self.dest01)
        # make sure no h5 exists yet
        for f in os.listdir(dataDir01):
            assert not f.lower().endswith(".h5")
        self.dest10 = os.path.join(self.inputDir, os.path.basename(dataDir10))
        for f in os.listdir(dataDir10):
            assert not f.lower().endswith(".h5")
        shutil.copytree(src=dataDir10, dst=self.dest10)

        # if needed create some h5 file from structure
        data_and_dir = (data01H5, self.dest01), (data10H5, self.dest10)
        for data, dataDir in data_and_dir:
            if data is not None:
                ft = FastSetupAll()
                ft.structures = data
                path = os.path.join(dataDir, "reconsH5File.h5")
                ft.writeAll(path, 3.8)

        # update scanList
        self.scanListWidget.clear()
        self.scanListWidget.add(self.dest01)
        self.scanListWidget.add(self.dest10)

    def clearInputFolder(self):
        for subFolder in os.listdir(self.inputDir):
            folder = os.path.join(self.inputDir, subFolder)
            assert os.path.isdir(folder)
            # should never store something else than a folder
            shutil.rmtree(folder)

    def setH5Exploration(self, b):
        """Activate or not the exploration"""
        self.ftserieWidget.setH5Exploration(b)

    def runAndTestList(self, structures, results, caseMsg):
        """Check that the reconstruction are send in the same order as the list
        is sending them
        """
        # 1.0 processing
        self.initData(structures[0], structures[1])
        self.scanListWidget._sendList()

        # first folder
        self.processOrangeEvents()
        # # second folder
        # self.processOrangeEvents()

        # 2.0 testing
        # check the first scan is correct
        output01 = os.path.join(self.dest01, _ReconsFtSeriesThread.copyH5ReconsName)
        self.assertTrue(os.path.isfile(output01))
        ft01 = FastSetupAll()
        ft01.readAll(output01, 3.8)

        # with self.subTest(msg=name):
        with self.subTest(
            msg="test reconstruction parameters used - folder1 : " + caseMsg,
            value=int(ft01.structures["FT"]["NUM_PART"]),
            expected=int(results[0]["FT"]["NUM_PART"]),
        ):
            self.assertEqual(
                int(ft01.structures["FT"]["NUM_PART"]),
                int(results[0]["FT"]["NUM_PART"]),
            )
        os.remove(output01)

        # check the second scan is correct
        output10 = os.path.join(self.dest10, _ReconsFtSeriesThread.copyH5ReconsName)
        self.assertTrue(os.path.isfile(output10))
        ft10 = FastSetupAll()
        ft10.readAll(output10, 3.8)

        with self.subTest(
            msg="test reconstruction parameters used - folder2 : " + caseMsg,
            value=ft10.structures["FT"]["NUM_PART"],
            expected=results[1]["FT"]["NUM_PART"],
        ):
            self.assertTrue(
                ft10.structures["FT"]["NUM_PART"] == results[1]["FT"]["NUM_PART"]
            )
        os.remove(output10)

    def getTestMsg(self, ftserieStatus, structures, results):
        """Return the message fitting with the structures we are setting to
        test01 and test10 and to the restul we want"""
        msg = "FTSerieWidget status : " + ftserieStatus
        msg += "\nConfiguration of folders : "
        msg += "\n    - Folder 1 contains "
        msg += (
            "no .h5"
            if structures[0] is self.default
            else "h5 with "
            + ("st1" if structures[0] is self.st1 else "st2")
            + " structure"
        )
        msg += "\n    - Folder 2 contains "
        msg += (
            "no .h5"
            if structures[1] is self.default
            else "h5 with "
            + ("st1" if structures[1] is self.st1 else "st2")
            + " structure"
        )
        msg += "\nResults in folder should be :"
        msg += "\n   - for first folder :"
        msg += (
            "default"
            if results[0] is self.default
            else ("st1" if results[0] is self.st1 else "ft2")
        )
        msg += "\n   - for second folder :"
        msg += (
            "default"
            if results[1] is self.default
            else ("st1" if results[1] is self.st1 else "ft2")
        )
        return msg


@unittest.skipIf(skip_gui_test(), reason="skip gui test")
@unittest.skip("Fail on CI...")
class TestLocalReconstructions(OrangeWorflowTest):
    """test with three widgets : ScanList, FTSerieWidget and ImageStackViewerWidget.
    Make sure that when path with data set:
        - scanList is sending signals
        - FTSerieWidget is reconstructing and emitting a signal
        - viewer is displaying a set of data (receiving the information)
    """

    def setUp(self):
        OrangeWorflowTest.setUp(self)
        self.inputdir = tempfile.mkdtemp()

        # copy files directly
        MockEDF.fastMockAcquisition(self.inputdir)
        self.ftserieListener = SignalListener()
        self.ftserieWidget.sigScanReady.connect(self.ftserieListener)

    def tearDow(self):
        if os.path.isdir(self.inputdir):
            shutil.rmtree(self.inputdir)
        OrangeWorflowTest.tearDown(self)

    @classmethod
    def setUpClass(cls):
        OrangeWorflowTest.setUpClass()
        # create widgets
        cls.nodeScanList = cls.addWidget(
            cls, "orangecontrib.tomwer.widgets.control.DataListOW.DataListOW"
        )
        cls.nodeFTSerie = cls.addWidget(
            cls, "orangecontrib.tomwer.widgets.reconstruction.FtseriesOW.FtseriesOW"
        )
        cls.nodeViewer = cls.addWidget(
            cls,
            "orangecontrib.tomwer.widgets.visualization.ImageStackViewerOW.ImageStackViewerOW",
        )
        cls.processOrangeEvents(cls)

        cls.link(cls, cls.nodeScanList, "data", cls.nodeFTSerie, "data")
        cls.link(cls, cls.nodeFTSerie, "data", cls.nodeViewer, "data")
        cls.processOrangeEvents(cls)

        cls.scanListWidget = cls.getWidgetForNode(cls, cls.nodeScanList)
        cls.ftserieWidget = cls.getWidgetForNode(cls, cls.nodeFTSerie)
        cls.viewerWidget = cls.getWidgetForNode(cls, cls.nodeViewer)

        # Set we only want to simulate the reconstruction
        cls.ftserieWidget._ftserie.reconsStack.setMockMode(True)
        # cls.ftserieWidget._ftserie.setForceSync(True)

    @classmethod
    def tearDownClass(cls):
        for node in (cls.nodeScanList, cls.nodeFTSerie, cls.nodeViewer):
            cls.removeNode(cls, node)
        OrangeWorflowTest.tearDownClass()

    def test(self):
        """Make sure the workflow is valid and end on the data transfert"""
        # add the path to the directory
        self.scanListWidget.add(self.inputdir)
        self.assertTrue(self.ftserieWidget._ftserie._scan is None)

        self.assertTrue(self.viewerWidget.viewer.getCurrentScanFolder() == "")
        self.assertTrue(self.viewerWidget.viewer.ftseriereconstruction is None)
        self.app.processEvents()
        self.scanListWidget.show()
        self.qWaitForWindowExposed(self.scanListWidget)
        # start the workflow by sending the list of path
        self.scanListWidget._sendList()

        timeout = 5
        loop_duration = 0.01
        while self.ftserieListener.callCount() < 1 and timeout > 0:
            self.app.processEvents()
            time.sleep(loop_duration)
            timeout -= loop_duration

        if timeout <= 0:
            raise TimeoutError("viewer is never activated")

        self.app.processEvents()

        self.assertEqual(self.viewerWidget.viewer.getCurrentScanFolder(), self.inputdir)


def suite():
    test_suite = unittest.TestSuite()
    for ui in (TestScanListFTSerieWorkflow, TestLocalReconstructions):
        test_suite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(ui))
    return test_suite



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/

__authors__ = ["H.Payno"]
__license__ = "MIT"
__date__ = "02/08/2017"

import unittest

from . import test_axis
from . import test_conditions
from . import test_darkref
from . import test_data_listener
from . import test_datatransfer
from . import test_datawatcher
from . import test_lamino
from . import test_nabu
from . import test_pyhstcaller
from . import test_recons_params
from . import test_scanlist
from . import test_timer


def suite():
    test_suite = unittest.TestSuite()
    test_suite.addTest(test_axis.suite())
    test_suite.addTest(test_conditions.suite())
    test_suite.addTest(test_darkref.suite())
    test_suite.addTest(test_data_listener.suite())
    test_suite.addTest(test_datawatcher.suite())
    test_suite.addTest(test_lamino.suite())
    test_suite.addTests(test_nabu.suite())
    test_suite.addTest(test_pyhstcaller.suite())
    test_suite.addTest(test_recons_params.suite())
    test_suite.addTest(test_scanlist.suite())
    test_suite.addTest(test_timer.suite())

    return test_suite



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/

__authors__ = ["C. Nemoz", "H. Payno"]
__license__ = "MIT"
__date__ = "25/10/2016"


from ..utils import WidgetLongProcessing
from Orange.widgets import widget, gui
from Orange.widgets.settings import Setting
from Orange.widgets.widget import Output
from tomwer.core.scan.scanbase import TomwerScanBase
from tomwer.core.process.control.datawatcher.datawatcher import _DataWatcher
from tomwer.gui.control.datawatcher import DataWatcherWidget
from tomwer.web.client import OWClient
import functools
import logging

logger = logging.getLogger(__name__)


class DataWatcherOW(widget.OWWidget, OWClient, WidgetLongProcessing):
    """
    This widget is used to observe a selected folder and his sub-folders to
    detect if they are containing valid-finished acquisitions.
    """

    name = "data watcher"
    id = "orangecontrib.widgets.tomwer.datawatcherwidget.DataWatcherOW"
    description = (
        "The widget will observe folder and sub folders of a given"
        " path and waiting for acquisition to be ended."
        " The widget will infinitely wait until an acquisition is "
        "ended. If an acquisition is ended then a signal "
        "containing the folder path is emitted."
    )
    icon = "icons/datawatcher.svg"
    priority = 12
    category = "tomwer"
    keywords = ["tomography", "file", "tomwer", "observer", "datawatcher"]

    allows_cycle = True
    compress_signal = False

    want_main_area = True
    resizing_enabled = True

    folderObserved = Setting(str())

    DEFAULT_DIRECTORY = "/lbsram/data/visitor"

    assert len(_DataWatcher.inputs) == 0
    assert len(_DataWatcher.outputs) == 1

    class Outputs:
        data_out = Output(
            name=_DataWatcher.outputs[0].name,
            type=_DataWatcher.outputs[0].type,
            doc=_DataWatcher.outputs[0].doc,
        )

    def __init__(self, parent=None, displayAdvancement=True):
        """Simple class which will check advancement state of the acquisition
        for a specific folder

        :param parent: the parent widget
        """
        widget.OWWidget.__init__(self, parent)
        OWClient.__init__(self)
        WidgetLongProcessing.__init__(self)
        self._widget = DataWatcherWidget(parent=self)
        self._widget.setFolderObserved(self.folderObserved)

        self._box = gui.vBox(self.mainArea, self.name)
        layout = self._box.layout()
        layout.addWidget(self._widget)

        # signal / slot connection
        self._widget.sigFolderObservedChanged.connect(self._updateSettings)
        self._widget.sigScanReady.connect(self._sendSignal)
        callback_start = functools.partial(self.processing_state, True, "watching on")
        self._widget.sigObservationStart.connect(callback_start)
        callback_end = functools.partial(self.processing_state, False, "watching off")
        self._widget.sigObservationEnd.connect(callback_end)

    def _updateSettings(self):
        self.folderObserved = self._widget.getFolderObserved()

    @property
    def currentStatus(self):
        return self._widget.currentStatus

    @property
    def sigTMStatusChanged(self):
        return self._widget.sigTMStatusChanged

    def resetStatus(self):
        self._widget.resetStatus()

    def _sendSignal(self, scan):
        assert isinstance(scan, TomwerScanBase)
        self.Outputs.data_out.send(scan)

    def setFolderObserved(self, path):
        self._widget.setFolderObserved(path)

    def setObservation(self, b):
        self._widget.setObservation(b)

    def setTimeBreak(self, val):
        """
        Set the time break between two loop observation
        :param val: time (in sec)
        """
        self._widget.setWaitTimeBtwLoop(val)

    def startObservation(self):
        try:
            self.processing_info("processing")
        except Exception:
            pass
        self._widget.start()

    def stopObservation(self, succes=False):
        self._widget.stop(succes)
        try:
            self.Processing.clear()
        except Exception:
            pass



# coding: utf-8
# /*##########################################################################
# Copyright (C) 2016-2018 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ############################################################################*/
"""Launch unittests of the library"""

__authors__ = ["V. Valls"]
__license__ = "MIT"
__date__ = "12/01/2018"

import sys
import argparse
import logging
import unittest
import os
from tomwer.test.utils import skip_gui_test


class StreamHandlerUnittestReady(logging.StreamHandler):
    """The unittest class TestResult redefine sys.stdout/err to capture
    stdout/err from tests and to display them only when a test fail.

    This class allow to use unittest stdout-capture by using the last sys.stdout
    and not a cached one.
    """

    def emit(self, record):
        """
        :type record: logging.LogRecord
        """
        self.stream = sys.stderr
        super(StreamHandlerUnittestReady, self).emit(record)

    def flush(self):
        pass


def createBasicHandler():
    """Create the handler using the basic configuration"""
    hdlr = StreamHandlerUnittestReady()
    fs = logging.BASIC_FORMAT
    dfs = None
    fmt = logging.Formatter(fs, dfs)
    hdlr.setFormatter(fmt)
    return hdlr


# Use an handler compatible with unittests, else use_buffer is not working
for h in logging.root.handlers:
    logging.root.removeHandler(h)
logging.root.addHandler(createBasicHandler())
logging.captureWarnings(True)

_logger = logging.getLogger(__name__)
"""Module logger"""


class TextTestResultWithSkipList(unittest.TextTestResult):
    """Override default TextTestResult to display list of skipped tests at the
    end
    """

    def printErrors(self):
        unittest.TextTestResult.printErrors(self)
        # Print skipped tests at the end
        self.printErrorList("SKIPPED", self.skipped)


def main(argv):
    """
    Main function to launch the unittests as an application

    :param argv: Command line arguments
    :returns: exit status
    """
    from silx.test import utils

    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument(
        "-v",
        "--verbose",
        default=0,
        action="count",
        dest="verbose",
        help="Increase verbosity. Option -v prints additional "
        + "INFO messages. Use -vv for full verbosity, "
        + "including debug messages and test help strings.",
    )
    parser.add_argument(
        "--qt-binding",
        dest="qt_binding",
        default=None,
        help="Force using a Qt binding: 'PyQt5' or 'PySide2'",
    )
    parser.add_argument(
        "--web",
        dest="web_log",
        default=False,
        help="Force unit test to export his log to graylog'",
        action="store_true",
    )
    parser.add_argument(
        "--no-orange-workflows",
        dest="no_orange_workflows",
        help="Turn off the test of the orange workflows",
        default=False,
        action="store_true",
    )
    utils.test_options.add_parser_argument(parser)

    options = parser.parse_args(argv[1:])

    test_verbosity = 1
    use_buffer = True
    if options.verbose == 1:
        logging.root.setLevel(logging.INFO)
        _logger.info("Set log level: INFO")
        test_verbosity = 2
        use_buffer = False
    elif options.verbose > 1:
        logging.root.setLevel(logging.DEBUG)
        _logger.info("Set log level: DEBUG")
        test_verbosity = 2
        use_buffer = False

    os.environ["_TOMWER_NO_ORANGE_WORKFLOWS_UNIT_TESTS"] = str(
        options.no_orange_workflows
    )
    os.environ["_TOMWER_NO_GUI_UNIT_TESTS"] = str(not options.gui)
    if options.qt_binding and options.gui is True:
        binding = options.qt_binding.lower()
        if binding == "pyqt4":
            _logger.info("Force using PyQt4")
            import PyQt4.QtCore  # noqa
        elif binding == "pyqt5":
            _logger.info("Force using PyQt5")
            import PyQt5.QtCore  # noqa
        elif binding == "pyside":
            _logger.info("Force using PySide")
            import PySide.QtCore  # noqa
        elif binding == "pyside2":
            _logger.info("Force using PySide2")
            import PySide2.QtCore  # noqa

    previous_web_log_value = os.environ.get("ORANGE_WEB_LOG")
    os.environ["ORANGE_WEB_LOG"] = str(options.web_log)

    # Configure test options
    utils.test_options.configure(options)

    # Run the tests
    runnerArgs = {}
    runnerArgs["verbosity"] = test_verbosity
    runnerArgs["buffer"] = use_buffer
    runner = unittest.TextTestRunner(**runnerArgs)
    runner.resultclass = TextTestResultWithSkipList

    # Display the result when using CTRL-C
    unittest.installHandler()

    import tomwer.test

    test_suite = unittest.TestSuite()
    test_suite.addTest(tomwer.test.suite())

    if options.gui is True:
        import orangecontrib.tomwer.test

        test_suite.addTest(orangecontrib.tomwer.test.suite())
    result = runner.run(test_suite)

    # reset the if ORANGE_WEB_LOG value
    if previous_web_log_value is None:
        del os.environ["ORANGE_WEB_LOG"]
    else:
        os.environ["ORANGE_WEB_LOG"] = previous_web_log_value

    if result.wasSuccessful():
        exit_status = 0
    else:
        exit_status = 1
    return exit_status



import os
import sys

from silx.gui import qt
from Orange.canvas import config, __main__ as main
from Orange.misc import environ
from .splash import splash_screen, getIcon
import tomwer.version
from tomwer.gui import icons
from tomwer.core.log.logger import _formatter_message, _ColoredFormatter
from tomwer.web.config import grayport_host, grayport_port
import logging

try:
    import graypy

    has_graypy = True
except:
    has_graypy = False

_logger = logging.getLogger(__name__)


def version():
    return tomwer.version.version


class TomwerConfig(config.Config):
    ApplicationName = "tomwer"
    ApplicationVersion = version()

    @staticmethod
    def splash_screen():
        return splash_screen()

    @staticmethod
    def core_packages():
        return super(TomwerConfig, TomwerConfig).core_packages() + ["tomwer-add-on"]

    @staticmethod
    def application_icon():
        return getIcon()


class TomwerSplashScreen(qt.QSplashScreen):
    def __init__(
        self,
        parent=None,
        pixmap=None,
        textRect=None,
        textFormat=qt.Qt.PlainText,
        **kwargs
    ):
        super(TomwerSplashScreen, self).__init__(pixmap=icons.getQPixmap("tomwer"))

    def showMessage(self, message, alignment=qt.Qt.AlignLeft, color=qt.Qt.black):
        version = "tomwer version {}".format(tomwer.version.version)
        super().showMessage(version, qt.Qt.AlignLeft | qt.Qt.AlignBottom, qt.Qt.white)


class Launcher:
    """Proxy to orange-canvas"""

    def launch(self, argv):
        config.Config = TomwerConfig
        self.fix_application_dirs()
        self.replace_splash_screen()
        self.main(argv)

    def fix_application_dirs(self):
        def data_dir(versioned=True):
            """
            Return the platform dependent Orange data directory.

            This is ``data_dir_base()``/Orange/__VERSION__/ directory if versioned is
            `True` and ``data_dir_base()``/Orange/ otherwise.
            """
            base = environ.data_dir_base()
            if versioned:
                return os.path.join(base, "tomwer", version())
            else:
                return os.path.join(base, "tomwer")

        environ.data_dir = data_dir

        def cache_dir(*args):
            """
            Return the platform dependent Orange cache directory.
            """
            if sys.platform == "darwin":
                base = os.path.expanduser("~/Library/Caches")
            elif sys.platform == "win32":
                base = os.getenv("APPDATA", os.path.expanduser("~/AppData/Local"))
            elif os.name == "posix":
                base = os.getenv("XDG_CACHE_HOME", os.path.expanduser("~/.cache"))
            else:
                base = os.path.expanduser("~/.cache")

            base = os.path.join(base, "tomwer", version())
            if sys.platform == "win32":
                # On Windows cache and data dir are the same.
                # Microsoft suggest using a Cache subdirectory
                return os.path.join(base, "Cache")
            else:
                return base

        environ.cache_dir = cache_dir

    def replace_splash_screen(self):
        main.SplashScreen = TomwerSplashScreen

    def graylog_dau_is_available(self):
        import socket

        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        try:
            s.connect((grayport_host, grayport_port))
            s.shutdown(2)
            return True
        except:
            return False

    def add_graypy_handler(self, my_logger):
        if not self.graylog_dau_is_available():
            return

        if os.environ.get("ORANGE_WEB_LOG", "True") == "True":
            if hasattr(graypy, "GELFHandler"):
                constructor = graypy.GELFHandler
            elif hasattr(graypy, "GELFUDPHandler"):
                constructor = graypy.GELFUDPHandler
            else:
                _logger.warning("Fail to find the correct constructor from graypy")
                return

            try:
                self.graylogHandler = constructor(grayport_host, grayport_port)
            except:
                _logger.warning("Fail to create GELFHandler. Won't report log message")
            else:
                my_logger.addHandler(self.graylogHandler)
                _logger.debug("- add graypy handler")
        else:
            info = "No log will be send to graylog."
            info += "ORANGE_WEB_LOG variable is setted to False"
            _logger.debug(info)

    def main(self, argv):
        from Orange.canvas.__main__ import main
        import optparse

        if argv is None:
            argv = sys.argv

        usage = "usage: %prog [options] [workflow_file]"
        parser = optparse.OptionParser(usage=usage)

        parser.add_option(
            "--no-discovery",
            action="store_true",
            help="Don't run widget discovery " "(use full cache instead)",
        )
        parser.add_option(
            "--force-discovery",
            action="store_true",
            help="Force full widget discovery " "(invalidate cache)",
        )
        parser.add_option(
            "--load-widget-settings",
            dest="clear_widget_settings",
            action="store_false",
            default=True,
            help="Load widget setting stored",
        )
        parser.add_option(
            "--no-welcome", action="store_true", help="Don't show welcome dialog."
        )
        parser.add_option(
            "--no-splash", action="store_true", help="Don't show splash screen."
        )
        parser.add_option(
            "-l",
            "--log-level",
            help="Logging level (0, 1, 2, 3, 4)",
            type="int",
            default=1,
        )
        parser.add_option("--style", help="QStyle to use", type="str", default=None)
        parser.add_option(
            "--stylesheet",
            help="Application level CSS style sheet to use",
            type="str",
            default=None,
        )
        parser.add_option(
            "--qt",
            help="Additional arguments for QApplication",
            type="str",
            default=None,
        )
        parser.add_option(
            "--all-addon",
            action="store_true",
            help="display only add-on widgets",
            default=False,
        )
        parser.add_option(
            "--no-color-stdout-logs",
            "--no-colored-logs",
            action="store_true",
            help="instead of having logs in the log view, color logs of the stdout",
            default=False,
        )
        (options, args) = parser.parse_args(argv[1:])
        levels = [
            logging.CRITICAL,
            logging.ERROR,
            logging.WARN,
            logging.INFO,
            logging.DEBUG,
        ]
        log_level = min(levels[options.log_level], logging.WARNING)
        for logger_name in (
            "fabio",
            "tomoscan",
            "nxtomomill",
            "tomwer",
            "nxtomomill.converter",
            "orangecontrib",
            "silx",
        ):
            logger = logging.getLogger(logger_name)
            logger.setLevel(log_level)

        # update tomwer logger
        FORMAT = (
            "%(asctime)s [%(levelname)-18s] %(message)s [$BOLD%(name)-20s$RESET]"
            "($BOLD%(filename)s$RESET:%(lineno)d)"
        )
        self.color_format = _formatter_message(
            FORMAT,
            use_color=os.environ.get("ORANGE_COLOR_STDOUT_LOG", "False") == "True",
        )

        color_formatter = _ColoredFormatter(self.color_format)

        console = logging.StreamHandler()
        console.setFormatter(color_formatter)

        tomwer_logger = logging.getLogger("tomwer")

        if has_graypy:
            self.add_graypy_handler(tomwer_logger)
        else:
            logger.info("graypy not install. No log will be send to graylog")

        main(argv)
